#!/usr/bin/perl

# This script generates Subversion stats using StatSVN based on the two (2)
# build versions provided in the last two pulldown menus. I use dynamically
# populated pulldown menus when the first pulldown menu is changed. For example,
# if the user selects "Ares" from the 1st pulldown menu, the following 2 pulldowns
# are populated with Jenkins Ares build versions. This is done by a Jenkins API
# call that finds all the successful Ares build versions and uses these numbers
# to populate these menus. These build version numbers are used to create
# indivudual arrays that also contain the date/time of the build according to
# Jenkins. When the user clicks the submit button, these date/time values are
# used for the svn log query for StatSVN so the stats we get contain only data
# between these 2 time points. This essentially provides data between 2
# successful Jenkins builds for a project, i.e Ares, LeptonV2-7.0.0 etc.

use File::Basename;
use IO::Handle;
use LWP::Simple;

$ENV{JAVA_HOME}='/opt/java/jdk1.6.0_26';
$CURRENT_PATH="$ENV{PATH}";
$ENV{PATH}="$CURRENT_PATH:$ENV{JAVA_HOME}/bin";

$thisscript=basename($0);
($thisscript_strip_cgi_ext = $thisscript) =~ s/\.cgi//g;

# Just so I can copy the chnages made in the buildstat-test .cgi to
# buildstat.cgi
if ( "$thisscript" eq "buildstat.cgi" ) {
	$svnurl="https://subversion.bit9.local/svn";
	$viewvc_string="viewvc";
} else {
	$svnurl="https://subversion.bit9.local/svntest";
	$viewvc_string="viewvc-";
	# Force live SVN so I can pick up live data for test run
	$svnurl="https://subversion.bit9.local/svn";
	$viewvc_string="viewvc";
}

# I only support the Das repo at this time
$svnrepo="Das";
$thisdir=dirname($0);
$webstats_topdir="/opt/apps/Webstats";
$wwwDir="/opt/apps/Webstats";
$wwwURL="https://jenkins.bit9.local/WebStats";
# https://subversion.bit9.local/viewvc/Das?revision=46948&view=revision"
$viewVC="https://subversion.bit9.local/viewvc/$svnrepo";

##############################################################################
#
# Project specific settings. These will need to change over time.
#
# These files are created from the Jenkins disable-jenkins-jobs job that the
# full Ares and LeptonV2-7.0.0 builds call when they're done
$Release702_statsvn_db_file="/mnt/builds/releng/buildVersions/Release702_Parity_Windows_Server_and_Client.v2";
$Release702_Version_String="7\.0\.2\.";

$Ares_statsvn_db_file="/mnt/builds/releng/buildVersions/Ares_Parity_Windows_Server_and_Client.v2";
$Ares_Version_String="7\.0\.1\.";

$LeptonV2_statsvn_db_file="/mnt/builds/releng/buildVersions/LeptonV2-7.0.0_Parity_Windows_Server_and_Client.v2";
# Since I renamed the LeptonV2 branch to LeptonV2-7.0.0, this
# LeptonV2_Version_String does not work so I use the actual version string
# "7\.0\.0\." where it's needed
#$LeptonV2_Version_String="7\.0\.0\.";
##############################################################################

$DEBUG=1;
$debuglog="/tmp/$thisscript_strip_cgi_ext.out";
open(DEBUGOUT, ">>$debuglog") if ($DEBUG);
print DEBUGOUT "In $thisscript_strip_cgi_ext\n";

sub ReadParse {
	local (*in) = @_ if @_;
	local ($i, $loc, $key, $val);

	# Read in text
	if ($ENV{'REQUEST_METHOD'} eq "GET") {
		$in = $ENV{'QUERY_STRING'};
	} elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
		read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
	}

	@in = split(/&/,$in);

	foreach $i (0 .. $#in) {
		# Convert pluses to spaces
		$in[$i] =~ s/\+/ /g;

		# Split into key and value.
		($key, $val) = split(/=/,$in[$i],2); # splits on the first =.

		# Convert %XX from hex numbers to alphanumeric
		$key =~ s/%(..)/pack("c",hex($1))/ge;
		$val =~ s/%(..)/pack("c",hex($1))/ge;

		# Associate key and value
		$in{$key} .= "\0" if (defined($in{$key})); # \0 is the multiple separator
		$in{$key} .= $val;

		print DEBUGOUT "AAA key is \"$key\"\n" if ($DEBUG);
		print DEBUGOUT "AAA val is \"$val\"\n" if ($DEBUG);

		if ( $key =~ /optone/ ) {
			$optone="$val";
			print DEBUGOUT "optone is \"$optone\"\n" if ($DEBUG);
		} elsif ( $key =~ /opttwo/ ) {
			$fromVersion="$val";
			print DEBUGOUT "fromVersion is \"$fromVersion\"\n" if ($DEBUG);
		} elsif ( $key =~ /optthree/ ) {
			$toVersion="$val";
			print DEBUGOUT "toVersion is \"$toVersion\"\n" if ($DEBUG);
		}
	}
}

# Generate the @ares_array with the Jenkins build version for javascript to use
# The disable-jenkins-jobs creates the DB files for Ares and LeptonV2-7.0.0

@out=`grep -A1 \" $Ares_Version_String" $Ares_statsvn_db_file`;
chomp(@out);
$numofelements=scalar(@out);
$numofelements--;
print DEBUGOUT "\'$numofelements\',\n";
$num=0;
$skip_version=0;
foreach $line (@out) {
	$num++;
        if ($skip_version) {
                $skip_version=0;
                next;
        }
	if ( @ares_array =~ m/./ ) {
		%seen=(); @unique_ares_array = grep { ! $seen{$_} ++ } @ares_array;
		@ares_array=();
		push(@ares_array, @unique_ares_array);
	}

	print DEBUGOUT "In XXX2 line is $line\n";
        if ( "$line" =~ m/\.X/ ) {
                $skip_version=1;
                next;
        }
        #if ( "$line" =~ m/7\.0\.1\./ ) {
        if ( "$line" =~ m/$Ares_Version_String/ ) {
                $line =~ s/\s+//g; #remove all space characters
        	if ( "$line" =~ m/\-/ ) {
                	$skip_version=1;
                	next;
        	}
                $version="$line";
		if ($num >= $numofelements) {
			$string = "'" . "$version" . "'";
		} else {
			$string = "'" . "$version" . "',";
		}
		push(@ares_array, $string);
                next;
        } else {
                $line =~ m/.*?(\">).*?(\">)(.*?)</;
                $dateField=$3;
                next if ( "$dateField" !~ m/./ );
                # date -d "Apr 18, 2012 10:56:37 AM" "+%Y-%m-%d %H:%M:%S"
                $svndatestr=`date -d "$dateField" "+%Y-%m-%d %H:%M:%S"`;
                chomp($svndatestr);
		if ( "@$version" !~ m/./ ) {
			push(@$version, "$svndatestr");
			print DEBUGOUT "Version: $version Date: $svndatestr\n" if ($DEBUG);
		}
        }
}

@out=`grep -A1 \" $Release702_Version_String" $Release702_statsvn_db_file`;
chomp(@out);
$numofelements=scalar(@out);
$numofelements--;
print DEBUGOUT "\'$numofelements\',\n";
$num=0;
$skip_version=0;
foreach $line (@out) {
	$num++;
        if ($skip_version) {
                $skip_version=0;
                next;
        }
	if ( @release702_array =~ m/./ ) {
		%seen=(); @unique_release702_array = grep { ! $seen{$_} ++ } @release702_array;
		@release702_array=();
		push(@release702_array, @unique_release702_array);
	}

	print DEBUGOUT "In XXX2 line is $line\n";
        if ( "$line" =~ m/\.X/ ) {
                $skip_version=1;
                next;
        }
        #if ( "$line" =~ m/7\.0\.1\./ ) {
        if ( "$line" =~ m/$Release702_Version_String/ ) {
                $line =~ s/\s+//g; #remove all space characters
        	if ( "$line" =~ m/\-/ ) {
                	$skip_version=1;
                	next;
        	}
                $version="$line";
		if ($num >= $numofelements) {
			$string = "'" . "$version" . "'";
		} else {
			$string = "'" . "$version" . "',";
		}
		push(@release702_array, $string);
                next;
        } else {
                $line =~ m/.*?(\">).*?(\">)(.*?)</;
                $dateField=$3;
                next if ( "$dateField" !~ m/./ );
                # date -d "Apr 18, 2012 10:56:37 AM" "+%Y-%m-%d %H:%M:%S"
                $svndatestr=`date -d "$dateField" "+%Y-%m-%d %H:%M:%S"`;
                chomp($svndatestr);
		if ( "@$version" !~ m/./ ) {
			push(@$version, "$svndatestr");
			print DEBUGOUT "Version: $version Date: $svndatestr\n" if ($DEBUG);
		}
        }
}

@out=`grep -A1 \" 7\.0\.0\." $LeptonV2_statsvn_db_file`;
chomp(@out);
$numofelements=scalar(@out);
$numofelements--;
print DEBUGOUT "\'$numofelements\',\n";
$num=0;
$skip_version=0;
foreach $line (@out) {
	$num++;
        if ($skip_version) {
                $skip_version=0;
                next;
        }
	if ( @lepton_array =~ m/./ ) {
		%seen=(); @unique_lepton_array = grep { ! $seen{$_} ++ } @lepton_array;
		@lepton_array=();
		push(@lepton_array, @unique_lepton_array);
	}
	print DEBUGOUT "In XXX2 line is $line\n";
        if ( "$line" =~ m/\.X/ ) {
                $skip_version=1;
                next;
        }
        if ( "$line" =~ m/7\.0\.0\./ ) {
                $line =~ s/\s+//g; #remove all space characters
        	if ( "$line" =~ m/\-/ ) {
                	$skip_version=1;
                	next;
        	}
                $version="$line";
		if ($num >= $numofelements) {
			$string = "'" . "$version" . "'";
		} else {
			$string = "'" . "$version" . "',";
		}
		push(@lepton_array, $string);
                next;
        } else {
                $line =~ m/.*?(\">).*?(\">)(.*?)</;
                $dateField=$3;
                next if ( "$dateField" !~ m/./ );
                # date -d "Apr 18, 2012 10:56:37 AM" "+%Y-%m-%d %H:%M:%S"
                $svndatestr=`date -d "$dateField" "+%Y-%m-%d %H:%M:%S"`;
                chomp($svndatestr);
		push(@$version, "$svndatestr");
        }
}

sub doHeader {
	# Do not put this line in the print EOF section
	print "Content-type:text/html\r\n\r\n";

print<<EOF;

<html>
<head>
<title>Summary Filelist Between Subversion Branches</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"/>
<link rel="stylesheet" href="buildstat.css" type="text/css"/>

EOF

print<<EOF;
</head>
<body>

<p><img src="https://jenkins.bit9.local/buildstat-header-pic.jpg" width="600" height=160>
<img src="https://jenkins.bit9.local/buildstat.jpg" width="160" height=160>
<p>

<!-- From http://javascript.about.com/library/bl3drop.htm -->

<script type="text/javascript">
function setOptions(chosen, selbox) {

        //document.writeln(chosen)

	if (chosen == "Ares") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@ares_array);
		}
	} else if (chosen == "Release702") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@release702_array);
		}
	} else if (chosen == "LeptonV2") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@lepton_array);
		}
	}

	// Chome dupes menus so I need to know what browser is being used
	var str = navigator.appVersion;

        selbox.options.length = 0;
        if (chosen == " ") {
                selbox.options[selbox.options.length] = new Option('Please select one of the options above first',' ');
                setTimeout(setOptions(' ',document.myform.optthree),5);
        }
        if (chosen == "Ares") {
		if (! str.match(/Chrome/g)) {
                	for (i = 0; i < window.version.length; i++) {
                        	selbox.options[selbox.options.length] = new Option(window.version[i],window.version[i]);
                	}
		}
                setTimeout(setOptions('11',document.myform.optthree),5);
        }
        if (chosen == "LeptonV2") {
		if (! str.match(/Chrome/g)) {
                	for (i = 0; i < window.version.length; i++) {
                        	selbox.options[selbox.options.length] = new Option(window.version[i],window.version[i]);
                	}
		}
                setTimeout(setOptions('21',document.myform.optthree),5);
        }

        if (chosen == "Release702") {
		if (! str.match(/Chrome/g)) {
                	for (i = 0; i < window.version.length; i++) {
                        	selbox.options[selbox.options.length] = new Option(window.version[i],window.version[i]);
                	}
		}
                setTimeout(setOptions('21',document.myform.optthree),5);
        }

	for (i = 0; i < window.version.length; i++) {
		selbox.options[selbox.options.length] = new Option(window.version[i],window.version[i]);
	}
}
</script>
<div class="section">
<table border="1" width="90%">

<form name="myform" action="https://jenkins.bit9.local/$thisscript_strip_cgi_ext" method="POST">

<div class="section">
<h2>Choose the Jenkins Plan you would like to generate a report for</h2>

<select name="optone" size="1"
onchange="setOptions(document.myform.optone.options[document.myform.optone.selectedIndex].value,document.myform.opttwo);">
<option value=" " selected="selected"> </option>
<option value="Ares">Ares</option>
<option value="LeptonV2">LeptonV2-7.0.0</option>
<option value="Release702">Release702</option>
</select>: Pick Project to populate pulldown menus below. You cannot toggle between projects. You must refresh the page if you do so the correct versions are displayed<br> <br>

<select name="opttwo" size="1"
onchange="setOptions(document.myform.opttwo.options[document.myform.opttwo.selectedIndex].value,document.myform.optthree);">
<option value=" " selected="selected">Please select one of the options above first</option>
</select>: From Version<br> <br>

<select name="optthree" size="1">
<option value=" " selected="selected">Please select one of the options above first</option>
</select>: To Version

<input type="submit" value="Submit"> (Once your report is complete, your web browser will automatically be redirected)<br>

</form>

<br><br><font color="red"><b>Additional Information:</b></font><br><br>
<li>The <b>Das</b> SVN repository is only processed for this data. Documentation, ThirdParty and version.h are excluded from these reports<br>
<li>If you make a choice in the first menu and want to change your pick, <font color="red"><b>you must refresh</b></font> this page to start again<br>
<li>If you're the first query of the day, your request could take 1 minute to process and redirect to the newly generated report. Please be patient<br>
<li>The wider the version range, the longer it will take StatSVN to generate its data. Please be patient<br>
<li>If someone else performed the same request, you will be directed instantly to the previously cached report statistics<br>
<li>If the previous conditions do not apply, most queries take approx. 20 seconds to complete.

EOF

}

# I redirect STDOUT to a file so I can process the file and put info at the top
# of the output. For me to get the info I need, the script needs to run first so
# I can't just print to STDOUT directly.

$TF1="/tmp/tf1$$";
$TF2="/tmp/tf2$$";
$RF1="/tmp/rf1$$"; # Used for reverse file of svn log output

# Save original STDOUT
open (ORIGSTDOUT, ">&STDOUT") or die "Couldn't dup STDOUT: $!";
open (STDOUT, ">> $TF1") or die "Can't open $TF1: $!";
$stdout_restored="no";

print DEBUGOUT "ENV{'REQUEST_METHOD'} is \"$ENV{'REQUEST_METHOD'}\"\n" if ($DEBUG);

if ( $ENV{'REQUEST_METHOD'} ) {
	&doHeader;

	&ReadParse;

	$webdir="$webstats_topdir" . "/$fromVersion-$toVersion";
	$wwwPage="$wwwURL/$fromVersion-$toVersion";

	if ($DEBUG) {
		print DEBUGOUT "OOO fromVersion array is \"@$fromVersion\"\n";
		print DEBUGOUT "OOO toVersion array is \"@$toVersion\"\n";
		print DEBUGOUT "OOO webdir array is \"$webdir\"\n";
		print DEBUGOUT "OOO wwwPage array is \"$wwwPage\"\n";
	}

	$color="odd";
	if ( "$optone" !~ m/./ ) {
		print "</table></div></body></html>\n";
		#print "</body></html>\n";
	} elsif ( $fromVersion eq $toVersion ) {
		print "<br><br><font color=\"red\"><b>ERROR:</b></font> I Don't think so. Why would you ever want to compare a build against itself? From version $fromVersion and To version $toVersion cannot be the same<br>\n";
		print "</body></html>\n";
	} elsif ( -r "$webdir" ) {
		close STDOUT;
		# Restore original STDOUT 
		$stdout_restored="yes";
		open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!"; 
		close ORIGSTDOUT; 
		#unlink("$TF1");
		close(DEBUGOUT) if ($DEBUG);
		print "Location: $wwwPage\n\n";
	} elsif ( -f "/tmp/BUILDSTAT-RUNNING.$optone" ) {
		print "<br><br><font color=\"red\"><b>ERROR:</b></font> Someone is currently generating a report for $optone. Please try again in 30 seconds and don't forget to refresh this page first!<br>\n";
		print "</body></html>\n";
	} elsif ( "$optone" =~ m/./ ) {
		system("touch /tmp/BUILDSTAT-RUNNING.$optone");
		$toBranchString="$optone";
		$jenkin_workspace="/opt/apps/jenkins/jobs/${optone}_Subversion_Webstats/workspace";
		$jenkin_workspace_das="$jenkin_workspace/$svnrepo";
		$webstatsTop="https://jenkins.bit9.local/${optone}_Subversion_Webstats";

		#if ($ENV{'REQUEST_METHOD'} eq "POST") {
			#print "<br><img src=\"https://jenkins.bit9.local/spinner.gif\">Generating report data. Please wait ...</p>\n";
	#<p>
		#}

		chdir "$jenkin_workspace_das";
		$pwd=`pwd`;
		chomp($pwd);

		if ($DEBUG) {
			print DEBUGOUT "PWD: $pwd\n";
			print DEBUGOUT "toBranchString $toBranchString\n";
			print DEBUGOUT "jenkin_workspace $jenkin_workspace\n";
			print DEBUGOUT "jenkin_workspace_das $jenkin_workspace_das\n";
			print DEBUGOUT "webstatsTop $webstatsTop\n";

			print DEBUGOUT "svn log -r\"{@$fromVersion}:{@$toVersion}\" --xml -v $svnurl/$svnrepo/release_branches/$toBranchString\n";
		}

		system("svn log -r\"{@$fromVersion}:{@$toVersion}\" --xml -v $svnurl/$svnrepo/release_branches/$toBranchString > /tmp/svnlog$$.out");

		print DEBUGOUT "java -jar $jenkin_workspace/build_scripts/statsvn-0.7.0.jar -disable-twitter-button -exclude 'Documentation/**;ThirdParty/**;**/version.h;**/Changelog.txt' /tmp/svnlog$$.out .\n";

		#unlink("index.html");
		system("rm -f *.html *.png");
		system("java -jar $jenkin_workspace/build_scripts/statsvn-0.7.0.jar -disable-twitter-button -exclude 'Documentation/**;ThirdParty/**;**/version.h;**/Changelog.txt' /tmp/svnlog$$.out .");

		# If index.html does not exist, then no changes were found
		# between builds
		if ( ! -r "index.html" ) {
			print DEBUGOUT "No index.html file created\n";

			print "<br><br><font color=\"red\"><b>NOTICE: </b></font>There were no differences found between the $fromVersion and $toVersion builds<br>\n";
			print "</body></html>\n";

			close STDOUT;
			open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!";
			close ORIGSTDOUT;
			system("cat $TF1 | sed s/FILES_CHANGED/$count/g");
			close(DEBUGOUT) if ($DEBUG);

			unlink("/tmp/BUILDSTAT-RUNNING.$optone");
			unlink("$TF1");
			exit(0);
		}

		# <Change Request:24725> - Part1
		# Find files changed between builds, get the first and last
		# change made to the file and create a ViewVC link to the diff
		# Create a separate page so these files/links can be listed

		# Create a new viewvc_diffs.html file to include as part of report
		# Grab the first 26 lines from commitlog.html file for headers
		system("head -26 commitlog.html > viewvc_diffs.html");
		system("head -26 commitlog.html > google_charts.html");
		system("sed -i \"s/Commit\ /ViewVC\ /g\" viewvc_diffs.html");
		system("sed -i \"s/Commit\ Log/Google Chart/g\" google_charts.html");

		# SVN log from oldest to newest versions
		#print DEBUGOUT "XXX svn log -r\"{@$fromVersion}:{@$toVersion}\" -v \"$svnurl/$svnrepo/release_branches/$toBranchString\" | grep \"release_branches\" | egrep -v \"Documentation|ThirdParty|version\.h\"\n";

		@fileschanged=`svn log -r\"{@$fromVersion}:{@$toVersion}\" -v \"$svnurl/$svnrepo/release_branches/$toBranchString\" | grep \"release_branches\" | egrep -v \"Documentation|ThirdParty|version\.h\"`;
		chomp(@fileschanged);

		foreach $file (@fileschanged) {
			$file =~ s/^\s+//;
			#print DEBUGOUT "XXX 1 file is \"$file\"\n" if ($DEBUG);
			#$file =~ s/.*?\/release_branches\/$toBranchString\/(.*)/\1/g;
			#$file =~ m/(.*)?\/release_branches\/$toBranchString\/(.*)/;
			if ( $file =~ m/\(from\ / ) {
				$file =~ m/(.*?)\/release_branches\/$toBranchString\/(.*?)\ \(from.*/;
				$action="$1";
				$svnfile="$2";
			} else {
				$file =~ m/(.*?)\/release_branches\/$toBranchString\/(.*)/;
				$action="$1";
				$svnfile="$2";
			}
			$action =~ s/\s+//g; #remove all space characters
			$svnfile = $action . "-" . $svnfile;
			#print DEBUGOUT "XXX action is \"$action\"\n" if ($DEBUG);
			#print DEBUGOUT "XXX svnfile is \"$svnfile\"\n" if ($DEBUG);
			push(@newfileschangedarray, $svnfile);
		}

		%seen=(); @fileschanged_unique_array = grep { ! $seen{$_} ++ } @newfileschangedarray;
		@fileschanged_unique_array=sort(@fileschanged_unique_array);

		system("egrep \"revision=|action=\" /tmp/svnlog$$.out > $TF2");
		# Create reversed file using tac
		system("/usr/bin/tac $TF2 > $RF1");
		# Add a dummy line at the end to complete the read logic for
		# reverse file
		system("echo DUMMYLINE >> $RF1");

		@fileschanged=`cat $TF2`;
		chomp(@fileschanged);
		@fileschangedreverse=`cat $RF1`;
		chomp(@fileschangedreverse);

		open(TOUT, ">>viewvc_diffs.html");
		print TOUT "<br>\n";

		print TOUT "<table border=\"1\" width=\"90%\">\n";
		print TOUT "<tr class=\"summaryperiod\">\n";
		print TOUT "<td width=\"1%\">Action</td>\n";
		print TOUT "<td width=\"12%\">Version(s)</td>\n";
		print TOUT "<td width=\"82%\">Subversion File/Directory</td></tr>\n";

		$count=0;
		$color="odd";
		OUTER: foreach $file (@fileschanged_unique_array) {
			# Skip certain files
			next if ( $file =~ m/Changelog.txt/ );
			$color="odd";
			$color="even" if ( $count % 2 );
			$count++;
			$file =~ m/(.*?)\-(.*)/;
			#print DEBUGOUT "XXX file is \"$file\"\n" if ($DEBUG);
			$action="$1";
			$svnfile="$2";
			#print DEBUGOUT "XXX svnfile is \"$svnfile\"\n" if ($DEBUG);
			#print DEBUGOUT "XXX action is \"$action\"\n" if ($DEBUG);
			$newest_version=();
			$oldest_version=();
			$filename=();

			#print DEBUGOUT "XXX fileschanged is \"@fileschanged\"\n" if ($DEBUG);
			# Find oldest version first
			INNER1: foreach $xmlline (@fileschanged) {
				#print DEBUGOUT "XXX xmlline is \"$xmlline\"\n" if ($DEBUG);
				#print DEBUGOUT "XXX action is \"$action\"\n" if ($DEBUG);
				if ( $xmlline =~ m/revision=/ ) {
					($oldest_version = $xmlline) =~ s/.*?\"(.*)\".*/\1/g;
				} elsif ( "$xmlline" =~ m@$svnfile@ ) {
					#print TOUT "<tr class=\"$color\"><td>$action</td>\n";
					#print TOUT "<td>$oldest_version</td>\n";
					print DEBUGOUT "Oldest version: $oldest_version for $svnfile\n";
					if ( $action eq "D" ) {
						print "DELETED : $svnfile\n";
						print TOUT "<tr class=\"$color\"><td>$action</td>\n";
						print TOUT "<td>$oldest_version</td>\n";
						#print TOUT "<td>$oldest_version</td>\n";
						print TOUT "<td><a href=\"$viewVC?view=revision&revision=$oldest_version\">$svnfile</a><br></td>\n";
						next OUTER;
					} elsif ( $action eq "A" ) {
						print "ADDED   : $svnfile\n";
						print TOUT "<tr class=\"$color\"><td>$action</td>\n";
						print TOUT "<td>$oldest_version</td>\n";
						print TOUT "<td><a href=\"$viewVC/release_branches/$toBranchString/$svnfile\">$svnfile</a><br></td>\n";
						next OUTER;
					} else {
						print "MODIFIED: $svnfile\n";
						last INNER1;
					}
				}
			}
			INNER2: foreach $xmlline (@fileschangedreverse) {
				if ( "$xmlline" =~ m@$svnfile@ ) {
					$filename="$svnfile";
					$newest_version=();
				} elsif ( $xmlline =~ m/revision=/ ) {
					($newest_version = $xmlline) =~ s/.*?\"(.*)\".*/\1/g;
				} elsif ($filename and $newest_version) {
					if ( $newest_version eq $oldest_version ) {
						#print "One version only\n";
						$oldest_version--;
					}
					print TOUT "<tr class=\"$color\"><td>$action</td>\n";
					print TOUT "<td>$oldest_version-$newest_version</td>\n";
					print TOUT "<td><a href=\"$viewVC/release_branches/$toBranchString/$svnfile?r1=$newest_version&r2=$oldest_version&pathrev=$newest_version\">$svnfile</a><br></td>\n";
					#print TOUT "<a href=\"$viewVC/release_branches/$toBranchString/$svnfile?r1=$newest_version&r2=$oldest_version&pathrev=$newest_version\">$svnfile</a><br>\n";
					next OUTER;
				}
			}
		}
		close(TOUT);

		# Grab the last 5 lines from commitlog.html file for footer
		system("tail -5 commitlog.html >> viewvc_diffs.html");

		unlink("$TF2");
		unlink("$RF1");

		# End <Change Request:24725> - Part1
 
		print "<br><p>\n";
		system("/bin/cp /tmp/svnlog$$.out /tmp/svnlog.out-SAVE2");
		unlink("/tmp/svnlog$$.out");

		print DEBUGOUT "sed index.html file with $fromVersion and $toVersion version info\n" if ($DEBUG);
		system("/bin/cp index.html index.html-SAVE");
		system("sed -i \"s/Development\ Statistics/Development\ Statistics\ From\ $fromVersion\ to\ $toVersion\ /g\" index.html");

		# Generate the Excel spreadsheet file
		unlink("$jenkin_workspace_das/chart_line.xls");

		# Use the perl script in the /mnt/builds/releng/metrics dir
		# if it exists, else cat the latest one from SVN
		if ( -r "/mnt/builds/releng/metrics/line-chart.pl" ) {
			$perlScript="/mnt/builds/releng/metrics/line-chart.pl";
		} else {
			system("svn cat $svnurl/Tools/Release_Engineering/metrics/line-chart.pl > /tmp/line-chart$$.pl");
			$perlScript="/tmp/line-chart$$.pl";
		}
		# optone is the project name; i.e Ares or LeptonV2-7.0.0
		print DEBUGOUT "Running: perl $perlScript $optone $fromVersion $toVersion\n" if ($DEBUG);
		@chartData=`perl $perlScript $optone $fromVersion $toVersion`;
		print DEBUGOUT "chartData array is \"@chartData\"\n" if ($DEBUG);
		unlink("/tmp/line-chart$$.pl");

		# Grab the last 5 lines from commitlog.html file for footer
		system("tail -5 commitlog.html >> google_charts.html");

		open(TIN, "google_charts.html");
		open(TOUT, ">google_charts.out");
		while ( $line = <TIN> ) {
			chomp $line;
			if ( "$line" =~ m/<head>/ ) {
				print TOUT "$line\n";
				print TOUT "    <script type=\"text/javascript\" src=\"https://www.google.com/jsapi\"></script>\n";
				print TOUT "    <script type=\"text/javascript\">\n";
				print TOUT "      google.load(\"visualization\", \"1\", {packages:[\"corechart\"]})\;\n";
				print TOUT "      google.setOnLoadCallback(drawChart)\;\n";
				print TOUT "      function drawChart() {\n";
				print TOUT "        var data = google.visualization.arrayToDataTable([\n";
				print TOUT "          ['LOC', 'Added', 'Deleted', 'Changed'],\n";
				if ( "@chartData" =~ m/Error:/ ) {
					print DEBUGOUT "Error running $perlScript\n" if ($DEBUG);
					#print TOUT "          [ERROR RETRIEVING BUILD DATA]\n";
					print TOUT "          ['0', 0, 0, 0]\n";
				} else {
					foreach $dataline (@chartData) {
    						chomp $dataline;
    						if ( $dataline =~ m/version_array:/ ) {
        						$dataline =~ s/.*?:(.*)/\1/g;
        						$dataline =~ s/^\s+//; #remove leading space
        						push(@version_array, $dataline);
    						} elsif ( $dataline =~ m/added_lines_array:/ ) {
        						$dataline =~ s/.*?:(.*)/\1/g;
        						$dataline =~ s/^\s+//; #remove leading space
        						push(@added_lines_array, $dataline);
    						} elsif ( $dataline =~ m/deleted_lines_array:/ ) {
        						$dataline =~ s/.*?:(.*)/\1/g;
        						$dataline =~ s/^\s+//; #remove leading space
        						push(@deleted_lines_array, $dataline);
    						} elsif ( $dataline =~ m/modified_lines_array:/ ) {
        						$dataline =~ s/.*?:(.*)/\1/g;
        						$dataline =~ s/^\s+//; #remove leading space
        						push(@modified_lines_array, $dataline);
    						}
					}

					@version_array = split ' ', $version_array[0];
					@added_lines_array = split ' ', $added_lines_array[0];
					@deleted_lines_array = split ' ', $deleted_lines_array[0];
					@modified_lines_array = split ' ', $modified_lines_array[0];

					$numofelements=scalar(@version_array);
					$num=0;
					foreach $dataentry (@version_array) {
    						chomp $dataentry;
    						print TOUT "        ['$dataentry', $added_lines_array[$num], $deleted_lines_array[$num], $modified_lines_array[$num]]";
    						$num++;
    						if ($num >= $numofelements) {
        						print TOUT "\n";
    						} else {
        						print TOUT ",\n";
    						}
					}
				}
				print TOUT "        ])\;\n";
				print TOUT "        var options = {\n";
				print TOUT "          title: 'LOC Changed, Added and Deleted Statistics',\n";
				print TOUT "          hAxis: {title: \"$optone Builds\", titleTextStyle: {color: 'red'}}\n";
				print TOUT "        }\;\n";
				print TOUT "        var chart = new google.visualization.ColumnChart(document.getElementById('chart_div'))\;\n";
				print TOUT "        chart.draw(data, options)\;\n";
				print TOUT "     }\n";
				print TOUT "  </script>\n";
			} elsif ( "$line" =~ m/<\/body>/ ) {
				print TOUT "    <div id=\"chart_div\" style=\"width: 900px\; height: 500px\;\"></div>\n";
				print TOUT "$line\n";
			} else {
				print TOUT "$line\n";
			}
		}
		close(TIN);
		close(TOUT);
		# Grab the last 5 lines from commitlog.html file for footer
		#system("tail -5 commitlog.html >> google_charts.out");
		print DEBUGOUT "Created google_charts.html file\n";
		system("/bin/cp google_charts.out google_charts.html");

		# <Change Request:24725> - Part2
		# Add our Bit9 ViewVC link to the reports. Addional reports can
		# be added here as well
		open(TIN, "index.html");
		open(TOUT, ">index.out");
		while ( $line = <TIN> ) {
			chomp $line;
			if ( "$line" =~ m/Commit\ Logs/ ) {
				print TOUT "$line\n";
				print TOUT "    <li><a href=\"viewvc_diffs.html\">ViewVC Diffs</a></li>\n";
				print TOUT "    <li><a href=\"chart_line.xls\">LOC Excel Line Chart</a></li>\n";
				print TOUT "    <li><a href=\"google_charts.html\">LOC Google Bar Chart</a></li>\n";
			} else {
				print TOUT "$line\n";
			}
		}
		print DEBUGOUT "Added ViewVC link to index.html file\n";
		close(TIN);
		close(TOUT);
		system("/bin/cp index.out index.html");
		# End <Change Request:24725> - Part2
		
		#system("sed -i \"s/Development\ Statistics/Development\ Statistics\ From\ $fromVersion\ to\ $toVersion\ /g\" index.html");

		# Add ViewVC links to SVN changeset versions
		# Get all changeset numbers
		print DEBUGOUT "Start modifying additional html files\n";
		#@htmlfiles=`ls commitlog.html dir_*.html user*.html`;
		@htmlfiles=`ls commitlog.html user*.html`;
		chomp(@htmlfiles);
		foreach $htmlfile (@htmlfiles) {
			open(TIN, "$htmlfile");
			open(TOUT, ">$htmlfile.out");
			while ($tin_line = <TIN>) {
				chomp $tin_line;
				if ( $tin_line !~ m/revisionNumberInner/ ) {
					print TOUT "$tin_line\n";
					next;
				}
				$tin_line =~ m/>([0-9].*?)</;
				$cset=$1;
				push(@csets, $cset);
				$tin_line =~ s/$cset/<a href=\"$viewVC\?revision=$cset\&view=revision\">$cset/g;
				print TOUT "$tin_line\n";
			}
			close(TIN);
			close(TOUT);
			system("/bin/mv $htmlfile.out $htmlfile");
		}
		print DEBUGOUT "Complete modifying additional html files\n";

		mkdir "$webdir";
		#system("/bin/ln -s $webdir $wwwDir");
		system("/bin/cp * $webdir");
		unlink("/tmp/BUILDSTAT-RUNNING.$optone");
		print DEBUGOUT "Remove /tmp/BUILDSTAT-RUNNING.$optone\n";

		close STDOUT;
		# Restore original STDOUT 
		$stdout_restored="yes";
		open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!"; 
		close ORIGSTDOUT; 
		#unlink("$TF1");
		close(DEBUGOUT) if ($DEBUG);
		print "Location: $wwwPage\n\n";
	}
	#print "</body></html>\n";
}

# Restore original STDOUT 
if ( $stdout_restored eq "no" ) {
	close STDOUT;
	open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!"; 
	close ORIGSTDOUT; 
}
system("cat $TF1 | sed s/FILES_CHANGED/$count/g");
unlink("$TF1");
close(DEBUGOUT) if ($DEBUG);

exit 0;
