#!/usr/bin/env python
import cgi
import cgitb; cgitb.enable() # give useful debugging help
import os
import re
import time
import subprocess
import tempfile
import json
from xml.dom.minidom import parseString, parse

JIRA_RE = re.compile(r'Jira:([A-Z]+-[0-9]+)', re.IGNORECASE)


def BuildNumber_key(a):
    return int(a['BuildNumber'].split('.')[-1])

def get_jira_info(svn_message):
    res = re.search(JIRA_RE, svn_message)
    if not res:
        return ""
    tickets = list(res.groups())
    links = []
    for ticket in tickets:
        link = 'https://jira.carbonblack.local/browse/%s' % ticket
        hyperlink = '<a href=%s>%s</a>' % (link, ticket)
        links.append(hyperlink)

    return '\n<br>\n'.join(links)

def get_svn_info(revision):
    if revision == 'HEAD':
        return ('', '', '')
    revision = int(revision)
    svn_url = 'https://subversion.bit9.local/svn/Das'
    info = subprocess.Popen(['svn', 'log', '--xml', '-r', str(revision), svn_url], stdout=subprocess.PIPE).communicate()[0]
    xml = parseString(info)
    logentry = xml.getElementsByTagName('logentry')[0]
    author = logentry.getElementsByTagName('author')[0].firstChild.nodeValue
    msg = logentry.getElementsByTagName('msg')[0].firstChild.nodeValue
    return (revision, author, msg)

def get_latest_build(branch, signing, config, product):
    assert signing in ['internal', 'external', '*']
    assert config in ['Release' , 'Debug']
    unix_signing = 'unsigned'
    if signing == 'external':
        unix_signing = 'signed'
    tfile = tempfile.NamedTemporaryFile()

    server_search_aql = 'items.find({"name":{"$eq": "ParityServerSetup.exe"}},{"path":{"$match": "cbp/product/server/windows/%s/%s/%s/*"}},{"repo":{"$eq": "carbonblack"}}).include("*", "@Revision")' % (signing, branch, config)
    winagent_search_aql = 'items.find({"name":{"$eq": "ParityHostAgent.msi"}},{"path":{"$match": "cbp/product/agent/windows/%s/%s/%s/*"}},{"repo":{"$eq": "carbonblack"}}).include("*", "@Revision")' % (signing, branch, config)
    rh6_agent_search_aql = 'items.find({"name":{"$eq": "Bit9Redhat6Install.bsx"}},{"path":{"$match": "cbp/product/agent/lnx-rhel6/%s/%s/%s/*"}},{"repo":{"$eq": "carbonblack"}}).include("*", "@Revision")' % (unix_signing, branch, config)
    rh7_agent_search_aql = 'items.find({"name":{"$eq": "Bit9Redhat7Install.bsx"}},{"path":{"$match": "cbp/product/agent/lnx-rhel7/%s/%s/%s/*"}},{"repo":{"$eq": "carbonblack"}}).include("*", "@Revision")' % (unix_signing, branch, config)
    osx_agent_search_aql = 'items.find({"name":{"$eq": "Bit9MacInstall.bsx"}},{"path":{"$match": "cbp/product/agent/osx/%s/%s/%s/*"}},{"repo":{"$eq": "carbonblack"}}).include("*", "@Revision")' % (unix_signing, branch, config)

    product = product.upper()
    if product == 'SERVER':
        tfile.file.write(server_search_aql)
    elif product == 'WINAGENT':
        tfile.file.write(winagent_search_aql)
    elif product == 'RH6AGENT':
        tfile.file.write(rh6_agent_search_aql)
    elif product == 'RH7AGENT':
        tfile.file.write(rh7_agent_search_aql)
    elif product == 'OSXAGENT':
        tfile.file.write(osx_agent_search_aql)
    else:
        raise ValueError("Unknown value for product: %s" % product)

    tfile.file.flush()
    devnull = open(os.devnull, 'w')
    builds = subprocess.Popen(['curl', '-X', 'POST', '-u',
     'build-readonly:AKCp2VokETVydLrFktqtwT5XM1vGHBaXis99BSVjyqz9RZZvkPDJ9cfDyR1Ke4PoSHhsrVYnt',
      '-d' '@%s' % tfile.name, 'https://artifactory-re.bit9.local/artifactory/api/search/aql',
      '--header', 'Content-Type: text/plain'], stderr=devnull, stdout=subprocess.PIPE).communicate()[0]
    tfile.file.close()
    builds = json.loads(builds)
    assert 'results' in builds.keys()
    blds = []
    for b in builds['results']:
        b[u'BuildNumber'] = b['path'].split('/')[-1]
        blds.append(b)
    if len(blds) == 0:
        return {}
    blds.sort(key=BuildNumber_key, reverse=True)
    b = blds[0]
    for prop in b['properties']:
        b[prop['key']] = prop['value']
    b.pop('properties', None)
    b.pop('virtual_repos', None)
    b.pop('type', None)
    b.pop('depth', None)
    b['url'] = 'https://artifactory-re.bit9.local/artifactory/' + b['repo'] + '/' + b['path'] + '/' + b['name']
    if b.has_key(u'Revision'):
        rev, author, msg = get_svn_info(b['Revision'])
        b['svn_author'] = author
        b['svn_msg'] = msg
        b['jira_ticket'] = get_jira_info(msg)
    return b

if __name__ == '__main__':

    print "Content-Type: application/json"
    #print "Content-Type: text/html"
    print ""

    branch = None
    signing = '*'
    config = None
    product = 'SERVER'
    fs = cgi.FieldStorage()
    for key in fs.keys():
        if key.lower() == 'branch':
            branch = fs[key].value
        elif key.lower() == 'signing':
            val = fs[key].value.lower()
            assert val in ['internal', 'external']
            signing = val
        elif key.lower() == 'config':
            val = fs[key].value.lower()
            assert val in ['release', 'debug']
            config = val[0].upper() + val[1:]
        elif key.lower() == 'product':
            val = fs[key].value.upper()
            assert val in ['SERVER', 'WINAGENT', 'RH6AGENT', 'RH7AGENT', 'OSXAGENT']
            product = val
    assert branch
    assert config
    latest = get_latest_build(branch, signing, config, product)
    print json.dumps(latest)
