#!/usr/bin/perl

use Sys::Hostname;
$host = hostname;

# Only admins can view and manage SVN users
@valid_admins=qw(sdaigle build karoyan jchiachiaretta mmorley kbusch mvictory mgutermuth gdavis dcuthbertson cvilla jfournier);

##################################
# To put offline for maintenance
##################################
$offline="no";
#$offline="yes";

$mylogo="https://$host.bit9.local/hotfixes-logo.jpg";
$mypic="https://$host.bit9.local/hotfixes-pic.jpg";
$mytitle="Sustaining Group Create and Reserve Hotfix Branch";
$mycss="hotfixes.css";

if ($offline eq "yes" ) {

print<<EOF;

<html>
<head>
<title>$mytitle</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"/>
<link rel="stylesheet" href="$mycss" type="text/css"/>
<body>
<p><img src="$mylogo" width="600" height=160>
<img src="$mypic" width="213" height=160></p>
<p>

<h1><font color="red">Temporarily down for maintenance</font></h1>
We expect to be up shortly. Thanks for your patience.
</body>
</html>

EOF

exit 1;
}

$admin=0;
foreach $valid_admin (@valid_admins) {
	$admin=1 if ( $ENV{REMOTE_USER} eq "$valid_admin" );
}

if (!$admin) {
	&doHeader;
	print "<br><font color=\"red\"><b>ERROR:</b> Don't be silly. You are not an authorized user for this utility. Cannot proceed with your request. Please see an Administrator for assistance.</font><br>";
	print "</body></html>\n";
	exit 1;
}

# LeptonV2-7.0.0 version.h hotfix numbers are one (1) above the actual HF number, unlike
# Ares-7.0.1-V2 are in sync with each other. If Enable_Lepton_Hack is set to 1, this
# will cause this script to add 1 to the version.h version. Set to 0 when/if these
# numbers get back into sync with each other. The Sustaining group should let you know
# if/when this happens.
$Enable_Lepton_Hack=1;

use File::Basename;
use IO::Handle;
use Data::Dumper;

$thisscript=basename($0);
$progdir=dirname($0);
($thisscript_strip_cgi_ext = $thisscript) =~ s/\.cgi//g;

# So I can change the test script before making it live
if ( "$thisscript" eq "next-available-hotfix.cgi" ) {
        $svn_branch_query_script="$progdir/print-next-available-hotfix-num.pl";
	$hotfix_branches_file="/mnt/transfer/hotfix-branches/hotfix-branches-live.txt";
	$patch_branches_file="/mnt/transfer/hotfix-branches/patch-branches-live.txt";
	$http_name="hotfixes";
	$mode="live";
} else {
        $svn_branch_query_script="$progdir/print-next-available-hotfix-num.pl";
	$hotfix_branches_file="/mnt/transfer/hotfix-branches/hotfix-branches-test.txt";
	$patch_branches_file="/mnt/transfer/hotfix-branches/patch-branches-test.txt";
	$http_name="hotfixes-test";
	$mode="test";
}

##########################
# START FORCE TEST SECTION
#
# Force test SVN "testrepos" until this is ready to go live. When this goes live,
# delete this section
#
#$hotfix_branches_file="/mnt/transfer/hotfix-branches/hotfix-branches-test.txt";
#$patch_branches_file="/mnt/transfer/hotfix-branches/patch-branches-test.txt";
#$mode="test";
#
# END FORCE TEST SECTION
##########################

@supporting_files = ($svn_branch_query_script, $hotfix_branches_file, $patch_branches_file);

# This takes too long to run so the Generate_SVN_Hotfix_Branches_Database Jenkins job
# created these database files every 15 minutes
if (0) {
$hotfix_branches_file="/tmp/hotfix$$.txt";
$patch_branches_file="/tmp/patch$$.txt";

system("svn ls svn+ssh://build\@subversion.bit9.local/var/svn/repos/Das/release_branches svn+ssh://build\@subversion.bit9.local/var/svn/repos/Das/release_branches/retired | grep HF | sort -u | sed -e \"s@\/@@g\" > $hotfix_branches_file");

system("svn ls svn+ssh://build\@subversion.bit9.local/var/svn/repos/Das/release_branches svn+ssh://build\@subversion.bit9.local/var/svn/repos/Das/release_branches/retired | grep \"\-P|\-RC|\-CD\" | grep -v Patch | sort -u | sed -e \"s@\/@@g\" | egrep -v \"Ares-7.0.1-P1|Ares-7.0.1-P2|Ares-7.0.1-CD\" > $patch_branches_file");
}

$DEBUG=1;
$debuglog="/tmp/$thisscript_strip_cgi_ext.out";
open(DEBUGOUT, ">>$debuglog") if ($DEBUG);
print DEBUGOUT "In $thisscript_strip_cgi_ext\n";
print DEBUGOUT "ENV{REQUEST_URI} is \"$ENV{REQUEST_URI}\"\n";
#print DEBUGOUT Dumper \%ENV;

sub printErrorAndDie() {
	print "<p><font color=red><b>ERROR: </b></font> $_[0]<p>\n";
	print "</body></html>\n";
	&restore_stdout;
	exit 1;
}

sub ReadParse {
	local (*in) = @_ if @_;
	local ($i, $loc, $key, $val);

	# Read in text
	if ($ENV{'REQUEST_METHOD'} eq "GET") {
		$in = $ENV{'QUERY_STRING'};
	} elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
		read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
	}

	@in = split(/&/,$in);

	#print DEBUGOUT "Now in ReadParse\n" if ($DEBUG);

	foreach $i (0 .. $#in) {
		print DEBUGOUT "Now in ReadParse foreach\n" if ($DEBUG);
		# Convert pluses to spaces
		$in[$i] =~ s/\+/ /g;

		# Split into key and value.
		($key, $val) = split(/=/,$in[$i],2); # splits on the first =.

		# Convert %XX from hex numbers to alphanumeric
		$key =~ s/%(..)/pack("c",hex($1))/ge;
		$val =~ s/%(..)/pack("c",hex($1))/ge;

		# Associate key and value
		$in{$key} .= "\0" if (defined($in{$key})); # \0 is the multiple separator
		$in{$key} .= $val;

		if ( $key =~ /TestTrackNumbers/ ) {
			$TestTrackNumbers="$val";
			#print DEBUGOUT "TestTrackNumbers is \"$TestTrackNumbers\"\n" if ($DEBUG);
		} elsif ( $key =~ /startingVersion/ ) {
			$startingVersion="$val";
			#print DEBUGOUT "startingVersion is \"$startingVersion\"\n" if ($DEBUG);
		} elsif ( $key =~ /newHotfixBranch/ ) {
			$newHotfixBranch="$val";
			#print DEBUGOUT "newHotfixBranch is \"$newHotfixBranch\"\n" if ($DEBUG);
		} elsif ( $key =~ /BranchFrom/ ) {
			$BranchFrom="$val";
			#print DEBUGOUT "BranchFrom is \"$BranchFrom\"\n" if ($DEBUG);
		} elsif ( $key =~ /SubversionCommitComment/ ) {
			$SubversionCommitComment="$val";
			#print DEBUGOUT "SubversionCommitComment is \"$SubversionCommitComment\"\n" if ($DEBUG);
		}
	}
	#print DEBUGOUT "Now in ReadParse after foreach\n" if ($DEBUG);
	#HF35 branch for CR 36297 based on LeptonV2-7.0.0-P13
	#$SubversionCommitComment="HF$startingVersion branch for CR 36297 based on LeptonV2-7.0.0-P13";
}

sub doHeader {
	# Do not put this line in the print EOF section
	print "Content-type:text/html\r\n\r\n";

print<<EOF;
<html>
<head>
<title>$mytitle</title>
<link rel="stylesheet" href="$mycss" type="text/css"/>
</head>
<body>

<p><img src="$mylogo" width="600" height=160>
<img src="$mypic" width="213" height=160>
<p>

EOF
}

sub doGetHeader {

print<<EOF;
<table border="1" width="90%">

<form name="myform" action="https://$host.bit9.local/$http_name?confirmConfig" method="POST">

<h2>Create Hotfix branches from Patch branches to fix specific customer issues</h2>

Bit9 Platform <b><i>Branch-From</b></i> Version :
<SELECT NAME="BranchFrom" >

EOF

foreach $branch (@sorted_patch_branches) {
 print "<option value=\"$branch\">$branch</option>\n";
}

# Do not enable this. This is for creating hotfix branch off of hotfix branches and it
# is will not work with the current supporting script print-next-available-hotfix-num.pl
#foreach $branch (@sorted_hotfix_branches) {
 #print "<option value=\"$branch\">$branch</option>\n";
#}

print<<EOF;
</SELECT>
(Name of the SVN <i><b>Patch Branch</b></i> you want to branch from)

<p>
TestTrack Change Request numbers :
<INPUT NAME="TestTrackNumbers" size="50">
</SELECT>
(Multiple TestTrack numbers must be comma or space separated)

<!--
<p>
Branch Subversion Commit Comment:
<INPUT NAME="SubversionCommitComment" size="50">
</SELECT>
(The Subverison commit comment to be used when creating the SVN branches)
-->

<p>
<input type="submit" value="Submit">
</form>

EOF

}

sub doApplyConfig {

	# These variables should be set by the time this subroutine is called
	# TestTrackNumbers=$TestTrackNumbers
	# BranchFrom=$BranchFrom
	# newHotfixBranch=$newHotfixBranch
	# SubversionCommitComment=$SubversionCommitComment

	@necessary_arguments=();
	push (@necessary_arguments, "$TestTrackNumbers", "$startingVersion", "$BranchFrom", "$newHotfixBranch", "$SubversionCommitComment");

	# Force redirect to main page if variables aren't set
	foreach $necessary_argument (@necessary_arguments) {
		if ( "$necessary_argument" !~ m/./ ) {
			print "Content-Type: text/html\n\n";
			print "<META HTTP-EQUIV=refresh CONTENT=\"0;URL=https://$host.bit9.local/$http_name\">\n";
		}
	}

	#$htmlSubversionCommitComment="$SubversionCommitComment";

	#$newHotfixBranch =~ m/.*?-HF(.*?)-CR/;
	#$branchNumber = "$1";
	#$startingVersion="1" . $branchNumber . "00";

	print<<EOF;

<table border="1" width="90%">

<h2>Your Hotfix Branch Is Being Created</h2>

<table border="1">
<tr>
<td><u>Description</u></td>
<td><u>Values</u></td>
<td><u>Additional Information</u></td>
</tr>
<tr>
<td>New Hotfix Branch:</td>
<td><b>$newHotfixBranch</b></td>
<td>Your new Hotfix Branch Name</td>
</tr>
<tr>
<td>TestTrack Number(s):</td>
<td> <b>$TestTrackNumbers</b></td>
<td>TestTrack number(s) you supplied</td>
</tr>
<tr>
<td>Branch From:</td>
<td><b>$BranchFrom</b></td>
<td>Subversion branch point</td>
</tr>
<tr>
<td>Build version.h:</td>
<td><b>$startingVersion</b></td>
<td>The version.h file will be set to this</td>
</tr>
<tr>
<td>Subversion Commit Comment:</td>
<td><b>$SubversionCommitComment</b></td>
<td>SVN commit message</td>
</tr>
<tr>
<td>Your Email Address:</td>
<td><b>$ENV{REMOTE_USER}\@bit9.com</b></td>
<td>Email address you will receive final notification</td>
</tr>
</table>

<p>
<br>

<table border="1">
<tr>
<td>This will take a minute. You will receive an email when your new branch is ready for use.</td>
</tr>
</table>

EOF

print DEBUGOUT "wget -O /dev/null --no-check-certificate --auth-no-challenge --http-user=build --http-password=2a19f9d3a817e3eb49b154183be46104 https://jenkins.bit9.local/ci/job/Create_Hotfix_Branches/buildWithParameters?token=BIT9_TOKEN\&delay=0sec\&mode=$mode\&runby=$ENV{'REMOTE_USER'}\&startingVersion=$startingVersion\&newHotfixBranch=$newHotfixBranch\&BranchFrom=$BranchFrom\&SubversionCommitComment=$SubversionCommitComment\n" if ($DEBUG);

system("wget -O /dev/null --no-check-certificate --auth-no-challenge --http-user=build --http-password=2a19f9d3a817e3eb49b154183be46104 \"https://jenkins.bit9.local/ci/job/Create_Hotfix_Branches/buildWithParameters?token=BIT9_TOKEN\&delay=0sec\&mode=$mode\&runby=$ENV{'REMOTE_USER'}\&startingVersion=$startingVersion\&newHotfixBranch=$newHotfixBranch\&BranchFrom=$BranchFrom\&SubversionCommitComment=$SubversionCommitComment\"");

}

sub doPutHeader {

$newHotfixBranch=&showNewBranchName;

print DEBUGOUT "TestTrackNumbers is \"$TestTrackNumbers\"\n" if ($DEBUG);
print DEBUGOUT "BranchFrom is \"$BranchFrom\"\n" if ($DEBUG);
print DEBUGOUT "newHotfixBranch is \"$newHotfixBranch\"\n" if ($DEBUG);

# TestTrackNumbers is "28499-28663"
# BranchFrom is "Ares-7.0.1-V2-P10"
# newHotfixBranch is "Ares-7.0.1-V2-HF16-CR-28499-28663"

@necessary_arguments=();
#push (@necessary_arguments, "$TestTrackNumbers", "$BranchFrom", "$newHotfixBranch", "$SubversionCommitComment");
push (@necessary_arguments, "$TestTrackNumbers", "$BranchFrom", "$newHotfixBranch");

# Force redirect to main page if variables aren't set
foreach $necessary_argument (@necessary_arguments) {
	if ( "$necessary_argument" !~ m/./ ) {
		print "Content-Type: text/html\n\n";
		print "<META HTTP-EQUIV=refresh CONTENT=\"0;URL=https://$host.bit9.local/$http_name\">\n";
	}
}

$newHotfixBranch =~ m/.*?-HF(.*?)-CR/;
$branchNumber = "$1";
# Pad a leading zero if current HF# is less than 10
$branchNumber = sprintf("%2d", $branchNumber);
$branchNumber=~ tr/ /0/;

if ($Enable_Lepton_Hack and $newHotfixBranch =~ m/LeptonV2-7\.0\.0/ ) {
	#$LeptonV2_Version_Number = $branchNumber + 1;
	#$startingVersion="1" . $LeptonV2_Version_Number . "00";
	$startingVersion="5" . $branchNumber . "00";
} else {
	$startingVersion="1" . $branchNumber . "00";
}

$SubversionCommitComment="HF$branchNumber branch for CR(s) $TestTrackNumbers based on $BranchFrom";
print DEBUGOUT "startingVersion is \"$startingVersion\"\n" if ($DEBUG);
print DEBUGOUT "SubversionCommitComment is \"$SubversionCommitComment\"\n" if ($DEBUG);

print<<EOF;
<table border="1" width="90%">

<!-- <form name="myform" action="https://$host.bit9.local/$http_name?applyConfig\&TestTrackNumbers=$TestTrackNumbers\&BranchFrom=$BranchFrom\&newHotfixBranch=$newHotfixBranch\&SubversionCommitComment=$SubversionCommitComment" method="POST"> -->

<form name="myform" action="https://$host.bit9.local/$http_name?applyConfig" method="POST">

<h2>Your Hotfix Branch Configuration</h2>

<br>TestTrack Number(s): <b>$TestTrackNumbers</b>
<br>Branch From: <b>$BranchFrom</b>
<br>Build version.h: <b>$startingVersion</b>
<br>Subversion Commit Comment: <b>$SubversionCommitComment</b>
<br>Your Current Email Address: <b>$ENV{REMOTE_USER}\@bit9.com</b>

<br>
<br>New Hotfix Branch: <b>$newHotfixBranch</b> <- This will be your new Hotfix Branch Name
<p>
<font color=red><b>IMPORTANT:</font></b> Once you click the <b>Yes</b> button below, the following will happen:
<br><li>You will reserve the <b>$newHotfixBranch</b> SVN branch
<br><li>Your branches will be created in Subversion for the <b><i>Das</b></i>, <b><i>DasThirdParty</b></i>, <b><i>bit9installers</b></i> and <b><i>build</b></i> repositories, using the latest versions of the <b>$BranchFrom</b> branch as your branch point
<br><li>The Common/Include/version.h will be versioned to <b>$startingVersion</b>

<p>
<br>Please make absolutely sure your request is complete. If you need to change anything, click the <b>No</b> button.

<p>
<br>Are you sure you want to continue with your request?
<p>

<input type="hidden" name="TestTrackNumbers" value="$TestTrackNumbers" />
<input type="hidden" name="BranchFrom" value="$BranchFrom" />
<input type="hidden" name="startingVersion" value="$startingVersion" />
<input type="hidden" name="newHotfixBranch" value="$newHotfixBranch" />
<input type="hidden" name="SubversionCommitComment" value="$SubversionCommitComment" />

<input type="submit" value="Yes">
</form>

<form name="myform" action="https://$host.bit9.local/$http_name" method="POST">
<input type="submit" value="No">
</form>

EOF

}

sub restore_stdout {
	close STDOUT;
	# Restore original STDOUT 
	open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!"; 
	close ORIGSTDOUT; 
	#system("cat $TF1 | sed s/FILES_CHANGED/$count/g");
	system("cat $TF1");
	unlink("$TF1");
}

sub showNewBranchName {
	if ( "$TestTrackNumbers" =~ m/./ ) {

		# Replace commas with space characters if necessary
		$TestTrackNumbers =~ s/,/ /g;
		# Get rid of extra space chars
		$TestTrackNumbers =~ s/\ +/ /g;
		# Get rid of trailing slash if necessary
		if ( $TestTrackNumbers =~ m/\ $/ ) {
        		chop $TestTrackNumbers;
		}
		# Replace space chars with dashes if necessary
		$TestTrackNumbers =~ s/ /-/g;

		# Strip the PXX portion of the string
		#$BranchFrom =~ m/(.*)-P.*/;
		#$BranchFrom =~ m/(.*)?-P|RC.*/;
		$BranchFrom =~ m/(.*)(-P|-RC|-CD).*/;
		$mainVersion = "$1";

		print DEBUGOUT "perl $svn_branch_query_script $mainVersion \"$TestTrackNumbers\" $hotfix_branches_file\n" if ($DEBUG);
		$newHotfixBranchName=`perl $svn_branch_query_script $mainVersion \"$TestTrackNumbers\" $hotfix_branches_file`;
		chomp $newHotfixBranchName;

		if ( "$newHotfixBranchName" !~ m/./ ) {
			&printErrorAndDie("Call to $svn_branch_query_script returned unexpected value. Please see your administrator for assistance.");
		}

		print DEBUGOUT "newHotfixBranchName is \"$newHotfixBranchName\"\n" if ($DEBUG);
		return $newHotfixBranchName;
	}
}

# I redirect STDOUT to a file so I can process the file and put info at the top
# of the output. For me to get the info I need, the script needs to run first so
# I can't just print to STDOUT directly.

$TF1="/tmp/tf1$$";
#$TF1="/tmp/tf1ZZZ";
# Save original STDOUT
open (ORIGSTDOUT, ">&STDOUT") or die "Couldn't dup STDOUT: $!";
open (STDOUT, ">> $TF1") or die "Can't open $TF1: $!";

# This should never happen but if the /etc/httpd/conf.d/bit9-ssl.conf-EDITME file was
# fat fingered, its a possibility. Rerify the /hotfixes and /hotfixes-test sections
# are correct.
if ( "$ENV{REMOTE_USER}" !~ m/./ ) {
	&doHeader;
	&printErrorAndDie("REMOTE_USER is not set so I don't know who you are. Please see your administrator for assistance.");
}

foreach $supporting_file (@supporting_files) {
	if ( ! -e "$supporting_file" ) {
		&doHeader;
		&printErrorAndDie("$supporting_file file not found. Please see your administrator for assistance.");
	}
}

@patch_branches=`cat $patch_branches_file`;
chomp(@patch_branches);

# Sort strings with numbers in a natural ascending order
@ascending_sorted_patch_branches = @patch_branches[
    map { unpack "N", substr($_,-4) }
    sort
    map {
        my $key = $patch_branches[$_];
        $key =~ s[(\d+)][ pack "N", $1 ]ge;
        $key . pack "N", $_
    } 0..$#patch_branches
];
@sorted_patch_branches = reverse(@ascending_sorted_patch_branches);

# Do not enable this. This is for creating hotfix branch off of hotfix branches and it
# is will not work with the current supporting script print-next-available-hotfix-num.pl
#@hotfix_branches=`cat $hotfix_branches_file | egrep \"Ares|LeptonV2\"`;
#chomp(@hotfix_branches);
#@sorted_hotfix_branches = sort {$b cmp $a} @hotfix_branches;

print DEBUGOUT "ENV{'REQUEST_METHOD'} is \"$ENV{'REQUEST_METHOD'}\"\n" if ($DEBUG);

#if ( $ENV{'REQUEST_METHOD'} or $ENV{'REQUEST_METHOD'} eq "POST") {
#if ( $ENV{'REQUEST_METHOD'} ~= m/GET/ ) {

if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
	&doHeader;
	&doGetHeader;
	&ReadParse;
	print "</body></html>\n";
}

if ( $ENV{'REQUEST_METHOD'} eq "POST" and $ENV{REQUEST_URI} =~ m#confirmConfig# ) {
	print DEBUGOUT "Now in REQUEST_URI confirmConfig\n" if ($DEBUG);
	&doHeader;
	&ReadParse;
	&doPutHeader;
	print "</body></html>\n";
	&restore_stdout;
	exit(0);
}

if ( $ENV{'REQUEST_METHOD'} eq "POST" and $ENV{REQUEST_URI} =~ m#applyConfig# ) {
	print DEBUGOUT "Now in REQUEST_URI applyConfig\n" if ($DEBUG);
	&doHeader;
	&ReadParse;
	&doApplyConfig;
	print "</body></html>\n";
	&restore_stdout;
	exit(0);
}

if ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
	&doHeader;
	&ReadParse;
	&doGetHeader;
	print "</body></html>\n";
}

&restore_stdout;

$datenow=`date`;
chomp $datenow;
print DEBUGOUT "Run $datenow by: $ENV{REMOTE_USER}\n";
if ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
	print DEBUGOUT "TestTrack query: $TestTrackNumbers\n";
}
close(DEBUGOUT) if ($DEBUG);

exit 0;
