#!/usr/bin/perl

use File::Basename;
use Sys::Hostname;
use IO::Handle;

$host = hostname;

# All Engineering LDAP users can view this page, however, only admins can
# manage SVN users
@valid_admins=qw(sdaigle build jchiachiaretta karoyan jfournier rmeyer);

$admin=0;
foreach $valid_admin (@valid_admins) {
        $admin=1 if ( $ENV{REMOTE_USER} eq "$valid_admin" );
}

if (!$admin) {
        &doHeader;
        print "<br><font color=\"red\"><b>ERROR:</b> Don't be silly. You're not permitted to access this page.</font><br>";
        print "</body></html>\n";
        exit 1;
}

$thisscript=basename($0);
($thisscript_strip_cgi_ext = $thisscript) =~ s/\.cgi//g;

$date=`date +%m%d%y-%H%M%S`;
chomp $date;
$pass=`cat /var/www/.svnalert/.BiT9paSsIn0rD`;
chomp $pass;

sub doHeader {
        # Do not put this line in the print EOF section
        print "Content-type:text/html\r\n\r\n";

        print<<EOF;
<html>
<head>
<title>Release Engineering Information for Sustaining Group Only</title>
<link rel="stylesheet" href="bit9.css" type="text/css"/>
</head>
<body>

<!--
<p><img src="https://$host.bit9.local/Subversion-User-Administration.jpg" width="600" height=160>
<img src="https://$host.bit9.local/Subversion-User-Administration-Image.jpg" width="160" height=160>
<p>
-->

EOF
}

&doHeader;

print<<EOF;

<h1>Release Engineering Information for Sustaining Group Only</h1>

This page is to be used by the members of the Sustaining Group. It contains information about the Bit9 Software build environment for all projects.<br>This page should be used for reference when Sheldon is not around, so you know where things are and how to access them.
<p>
This does not go in-depth, but should give you enough information about building our products for official releases, updates and hotfixes, what systems<br>are used in the process and how to access them.
<p>

<h2>Release Engineering Info</h2>

ep>
	Use <a href="https://eng-re.bit9.local/buildinfo" target=_blank>https://eng-re.bit9.local/buildinfo</a> to know what projects/patches/updates we're building, what SVN branches are used for each and the Jenkins jobs associated with each build.
<p>
	Use the <a href="https://subversion.bit9.local/svnauth" target=_blank>https://subversion.bit9.local/svnauth</a> page to give/remove permissions to major SVN project and patch branches. Anything more complex, you need to login to the subversion server as user build and switch user to root and edit <b><i>/var/svn/conf/repos.acl</b></i> file. If you don't know what you're doing, get someone who does. One bad change can lock everyone out. If you mess this file up, delete the file and do an "svn update". All these files are in SVN. Use the build user account and the AD password for this user.
<p>
You should already have the local "build" user and "root" user passwords to these servers:
<p>
&nbsp;&nbsp;&nbsp;&bull; subversion.bit9.local  - subversion server
<br>
&nbsp;&nbsp;&nbsp;&bull; jenkins.bit9.local  - Engineering Jenkins server
<p>
<u><b>Jenkins</b></u>
<p>
The Jenkins application is served by <b><i>/etc/init.d/jenkins</b></i> on jenkins.bit9.local. This is the startup script and the application is accessed via  <a href="https://jenkins.bit9.local/ci" target=_blank>https://jenkins.bit9.local/ci</a>. This is started automatically at boot time, but if you need to manually restart the application, run "/etc/init.d/jenkins start" and wait approx. 3 minutes before trying to hit the <b><i>https://jenkins.bit9.local/ci</b></i> URL.
<p>
Jenkins Windows agents run as a Windows service. You must stop/start the Jenkins Windows service on the systems. Do not try to start the nodes from the Jenkins interface. Nodes rarely go offline, but if a build node is in an <b>offline</b> state, Remote Desktop to the Windows machine and manually Stop and Start the Jenkins process. Wait approx. 30 seconds to a minute and the build machine should come back online.
<p>
Jenkins Linux/MacOS agents are started by <b><i>"ssh"</b></i> connections from the Jenkins server by the <b><i>"mrcook"</b></i> user on the Engineering side and the <b><i>"build"</b></i> user in the "Offline" environment. All SSH keys have been configured to allow unchallenged login and startup script execution. Nodes rarely go offline, but if a Linux/Mac not is in an <b>offline</b> state, log into the Jenkins web inteface as user <b><i>"build"</b></i> with AD credentials, click on the node that's offline, and click Start to bring the node back online. This only takes a few seconds to complete.
<p>
<u><b>How Builds work</b></u>
<p>
	All Jenkins builds must be initiated from the Engineering Jenkins server <a href="https://jenkins.bit9.local/ci" target=_blank>https://jenkins.bit9.local/ci</a>
<p>
	All main builds can be found under the "<b><i>AG -> All Projects</b></i>". For the most part, the naming convention should be clear. The "Updates" and "Hotfixes" builds may not be that obvious. The "Updates" builds are used for Patch branch builds, i.e <b><i>Ares-7.0.1-V2-P7, LeptonV2-7.0.0-P12</b></i> etc. The "Hotfixes" builds should be used for hotfix branch builds. i.e <b><i>LeptonV2-7.0.0-HF31-CR-34811, Ares-7.0.1-V2-HF4-CR-34680</b></i> etc. Again, refer to the <a href="https://eng.bit9.local/buildinfo" target=_blank>https://eng-re.bit9.local/buildinfo</a> page for project information.
<p>
	All builds that do not code sign, are run on the Engineering side, i.e Continuous builds, Publish Releases, Push to SRS etc. All full builds that do code signing are passed over to the isolated build environment via a manifest.txt file that gets committed to SVN when the Engineering Jenkins job is run. The Isolated Jenkins server polls for these manifests and acts upon them. When the builds are complete, the Engineering Jenkins jobs, picks up the files/artifacts and finishes up.
<p>
	The laptop computer in the office, is the code signing machine. When builds run in the isolated environment, they package up all files that need to be signed, and the laptop signs them. You should never need to log on the laptop because it just processes jobs as they come. If for some reason you do need to access it, 
<p>
<u><b>Isolated build environment</b></u>
<p>
<b><font color=red>NEVER, NEVER, NEVER</font> change any network plugs or physical hardware configuration. If something needs to be looked at, Ask IT or SecOps for help. 
</b>
<p>
In my office, the monitors and keyboard against the wall access this environment. If everything is up and running, turn the monitors on, and login as user "build" with the standard password.
<p>
	The desktop PC (hostname porthole-ag) plays no part in the build process. It is configured to access the vSphere environment and Remote Desktop connections to all Windows build systems and VNC used for accessing all Linux and MacOS build systems.
<p>
	Use vSphere to access consoles of all build VMs and the isolated Jenkins server. The VMware vSphere icon is on the desktop. Login name is "releng" and the password is the same as the standard build user. You shouldn't need access to this unless the VMs are not accessible via Remote Desktop and/or SSH. This should never happen but just in case.
<p>
	Use Remote Desktop for accessing all Windows VMs. All Windows build VMs are named "windows-01" thru "windows-05". Remote Desktop should connect unchallenged, but if you need to supply credentials, use the build user and password.
<p>
	Use SSH for accessing all Linux and Mac build systems. The Linux systems are VMs and the Mac systems are physical Mac Minis. When you logon to the porthole-ag PC, there will be a VNC window open with many windows to other systems. You may use this if you like. If for some reason it's not running, or you prefer individual windows, there is a Cygwin Terminal icon you can use to access all systems. Open as many windows as you like.
<p>
<u><b>Linux system names:</b></u>
<p>
	&nbsp;&nbsp;&nbsp;&bull; redhat57-01 thru redhat57-03 - All Redhat 5.7 systems
<br>
	&nbsp;&nbsp;&nbsp;&bull; redhat58-01 thru redhat58-03 - All Redhat 5.8 systems
<br>
	&nbsp;&nbsp;&nbsp;&bull; redhat63-01 thru redhat63-03 - All Redhat 6.3 systems
<br>
	&nbsp;&nbsp;&nbsp;&bull; ubuntu10-01 thru ubuntu10-03 - All Ubuntu 10.04.3 systems
<p>
<u><b>Mac systems names:</b></u>
<p>
	&nbsp;&nbsp;&nbsp;&bull; mac-01 - MacOS 10.7.4
<br>
	&nbsp;&nbsp;&nbsp;&bull; mac-02 - MacOS 10.8.0
<br>
	&nbsp;&nbsp;&nbsp;&bull; mac-03 - MacOS 10.8.4 (Not being used until we get new Mac Signing cert) 
<p>

<u><b>How to Find and Read Build logs for Full, Signed builds</b></u>
<p>

The output for all Engineering side Jenkins builds can be found via the "Console Output" link. These builds do not sign anything, so they are still run on the Engineering side.
<p>
All builds that "Code Sign" are run in the offline build environment and their build logs can be found under the "Build Artifacts -> build-output" links for each build:
<p>
&nbsp;&nbsp;&nbsp;&bull; <b><i>WINDOWS-BUILDLOG.txt</b></i> - This is the Windows build output from all the python build scripts. This is a text version for the Jenkins console output from the job itself. i.e Ares-7.0.1-V2_Bit9_Platform, V720_Bit9_Platform. Since all non-Windows builds (linux/mac) run on the osolated build machines, we pass back the build logs to "build-output" artifacts so they can be viewed if necessary. Each non-Windows platform creates it's own subdirectory and extracts it logfiles. For example, for <b><i>Ares-7.0.1-V2</b></i> builds, you should see the following directory links:
<p>
&nbsp;&nbsp;&nbsp;&bull; <b><i>Ares-7.0.1-V2_Bit9_Platform_MacOS_Client-archive</b></i>
<br>
&nbsp;&nbsp;&nbsp;&bull; <b><i>Ares-7.0.1-V2_Bit9_Platform_RedHat57_Client-archive</b></i>
<br>
&nbsp;&nbsp;&nbsp;&bull; <b><i>Ares-7.0.1-V2_Bit9_Platform_RedHat58_Client-archive</b></i>
<br>
&nbsp;&nbsp;&nbsp;&bull; <b><i>Ares-7.0.1-V2_Bit9_Platform_RedHat63_Client-archive</b></i>
<br>
&nbsp;&nbsp;&nbsp;&bull; <b><i>Ares-7.0.1-V2_Bit9_Platform_Ubuntu10_Client-archive</b></i>
<br>
<p>
When initiating a build on the Engineering side, the job will wait 90 minutes or 2 hours (depending on the job), and if the offline environment did not pass back artifacts to the "<b><i>\\\\eng-file-server\\builds\\archives\\[BUILD_NUMBER]</b></i>" directory, the build will timeout and fail. 
<p>
These are the more difficult builds to diagnose and the easiest way is to use the monitors and keyboard against the wall in my office. Just use the Jenkins browser, locate the job you're looking for and go thru the logs.
<p>
If you Remote Desktop to a build machine, remember to close all DOS and File Explorer windows when you're done, so no file/directories are left opened causing locking issues for future builds.

<p>
<u><b>Creating branches (patches and hotfixes)</b></u>
<p>
<u>Patch Branches</u><br>
When creating <b>patch</b> branches for Bit9 Platform projects, I usually create and use script from the <a href="https://subversion.bit9.local/svn/Tools/Release_Engineering/misc-scripts/branch-tag-scripts" target=_blank>https://subversion.bit9.local/svn/Tools/Release_Engineering/misc-scripts/branch-tag-scripts</a> repository. You can execute "svn copy" commands to branch, but these script make things much easier and faster. For Bit9 Platform projects we usually branch the following repositories:
<p>
<li>Das
<li>DasThirdParty
<li>bit9installers
<p>
When creating "patch" branches, you must also do the following:
<p>
<b>1.</b>  Bump the mainline Das/Common/Include/version.h by 30 which should be more than enough and commit to SVN
<br>
<b>2.</b>  Bump bit9installers/Server/Server_Patch/Misc/PatchNumber.txt by 1 (I usually do "printf NN > PatchNumber.txt" ) and commit to SVN
<br>
<b>3.</b>  On the "subversion.bit9.local" system as user "root", edit the <b>/var/svn/conf/repos.acl</b> file and add the new branch sections to the file. For example if you're creating a new patch branch named "V720-P3", simply copy and paste the "Das", "DasThirdParty" and "bit9installer" sections for the previous "V720-P2" branch and change the V720-P2 strings to V720-P3. If there are individual developers listed under each section, remove them. The "svnauth" page (next step) will be used to add and remove users to this section of the auth file.
<br>
<b>4.</b>  On the "subversion.bit9.local" system as user "root", edit the <b>/var/www/cgi-bin/svnauth.cgi</b> file and add the new branch to the "@managedBranches" array near the top of the file. Also remove the old branches if they are no longer being used.
<br>
<b>5.</b>  Finally open the <a href="https://eng.bit9.local/buildinfo" target=_blank>https://eng.bit9.local/buildinfo</a> page and "Add New Release" and fill in the fields appropriately.
<p>
We do not usually branch the "build" repository, but there have been times when this needed to be done. For example when we needed to address the "Hearbleed" OpenSSL issues, the version of OpenSSL changed and the versions are hardcoded in the build scripts. This required the build scripts to be branched for some "patch" branches to pick up the correct versions of OpenSSL.
<p>
<u>Hotfix Branches</u><br>
When creating <b>hotfix</b> branches, the Sustaining group uses the <a href="https://jenkins.bit9.local/hotfixes" target=_blank>https://jenkins.bit9.local/hotfixes</a> page. Once branches are created, the appropriate Jenkins jobs should be used. Please refer to the <a href="https://eng.bit9.local/buildinfo" target=_blank>https://eng.bit9.local/buildinfo</a> page for current projects. This web pages will automatically modify the new branches "Common/Include/version.h" file.

<p>
<u><b>Miscellaneous</b></u>
<p>
Since installing Bit9 on all the Release Engineering servers and build machines, we've had a lot of problems. Development has been looking into these issues as they occur however, we continue to have problems. Management has decided to keep these systems in high enforcement so the only way to fix problems when they happen, is to stop and restart Bit9. If a build machine starts acting up, rebooting it is the easiest way to get it working again. When the Jenkins or Subversion servers have issues, you cannot reboot these because you will continuously disrupt Engineering.
<br>
<p>
 When the Subversion server is having problems, you will see may unexplained build failures in the currently running Jenkins jobs as well as errors in the server logs and people emailing and instant messaging asking if Subversion is down. 
<br>
<p>
When the Jenkins server is having issues, you will see unexplained build failures, jobs hanging etc.  A good indication that Bit9 is having issues is to execute <b>/opt/bit9/bin/b9cli -status</b> and if this hangs, you'll need to stop/start Bit9.
<br>
<p>
These problems can happen multiple times per day. To get the servers back to a good state, first try to stop and restart Bit9. If that doesn't work, notify Engineering you need to reboot the server and the do it. To Stop/Start Bit9, run these commands:
<br>
<p>
<li>/etc/init.d/b9daemon stop
<li>modprobe -r b9kernel
<li>modprobe b9kernel
<li>/etc/init.d/b9daemon start
<li>/opt/bit9/bin/b9cli -status
<p>

<b>
<b>
<p>

EOF

print "</body></html>\n";

exit 0;
