#!/usr/bin/perl

use DBI;
use File::Basename;

# SQL database fields info
# TargetBuild -  The complete build label
# TargetRelease  The release for which the build is associated
# State  An indicator of progress of the Smoke Test Run (Current state values include: Scheduled, Queued, Initializing, In Progress, Failed,  Completed,  Completed With Error
# AgentPlatform  The platform indicator that matches the Build Notification file
# AgentDescription  The readable description of the Agent Platform
# PassResultCount  The # of Test Cases that passed
# FailResultCount  The # of Test Cases that failed
# DetailsReportURL Links to detailed reports

# As of 8/14/2012:
# | RunID| RunName| TargetBuild | TargetRelease| State | AgentPlatform| AgentDescription| PassResultCount| FailResultCount| DetailsReportURL
# Example:
# | 161351f573782b1a3a1283aad0e38c1e | 7.0.1.440 - 08/14/12 - 15:15 | 7.0.1.440 | 7.0.1 | Completed | MAC | Mac Snow Leopard (10.7) 64-Bit Server | 1 | 7 | http://testtrack/ReportServer_TestTrack?%2fQA%2fTest+Results%2fTest+Run+-+Details&rs:Command=Render&TestRun=161351f573782b1a3a1283aad0e38c1e

# Some manual query examples from system jenkins:
# Configuration file: /etc/odbc.ini /etc/odbcinst.ini
# /usr/local/bin/isql TESTTRACK TestExecutionFramework TestExecutionFramework -v
# SQL> SELECT * FROM GetParityCISmokeTestStatus('7.0.1', '')

# Set to 1 for live TestTrack/TestTrack SQL database
$LIVEDB="1";

# testtype must be "ci" or "demand"
$testtype="$ARGV[0]";

$listonly=1 if ($ARGV[3]);

if ( $LIVEDB ) {
	# Used for live TestTrack instance
	$db_port="61933";
	#$db_server = "10.0.3.110";
	$db_server = "it-sql-2012.bit9.local";
	$db_name = "TestResults";
	$db_user = "TestExecutionFramework";
	$db_pass = "TestExecutionFramework";
} else {
	# Used for Adams TestTrack2 instance
	$db_port="52416";
	$db_server = "10.0.0.38";
	$db_name = "Test";
	$db_user = "ReadOnly";
	$db_pass = "ReadOnly";
}

sub getDateTime {
        $date=`date \"+%a %m-%d-%y %H:%M:%S\"`;
        chomp $date;
        $logprefix="$date - $ENV{REQUEST_URI} - $ENV{REMOTE_ADDR} - $ENV{REMOTE_USER}";
}

#$date=`date`;
#chomp($date);

$DEBUG=1;
$thisscript=basename($0);
($thisscript_strip_ext = $thisscript) =~ s/\.pl//g;
if ( "$thisscript" eq "smokestats-sqlquery.pl" ) {
	$debuglog="/tmp/smokestats-sqlquery.out";
} else {
	$debuglog="/tmp/smokestats-sqlquery-test.out";
}

$CONNECT = "DRIVER={SQL Server Native Client 11.0};Server=$db_server,$db_port;Database=$db_name;UID=$db_user;PWD=$db_pass";

$dbh = DBI-> connect("dbi:ODBC:$CONNECT");

# This query generates a result set with one record in it.
# These are examples of specific queries
# SELECT * FROM GetParityCISmokeTestStatus('7.0.1.341-sdaigle-13', '7.0.1')
# SELECT * FROM GetParityCISmokeTestStatus('7.0.1.341-sdaigle-13', '')
# SELECT * FROM [TestResults].[dbo].[GetParityCISmokeTestStatus] ('7.0.1', '')
# SELECT TargetBuild FROM [TestResults].[dbo].[GetParityBuildsWithCISmokeTestResults] ('7.0.1', '') ORDER BY TargetBuild DESC

# SELECT TargetBuild, TargetRelease, State, AgentPlatform, AgentDescription, PassResultCount, FailResultCount FROM [TestResults].[dbo].[GetParityCISmokeTestStatus] ('7.0.1', '')"

# I want to get all records and let this script parse the valid
if ( $LIVEDB ) {
	$build="$ARGV[1]";
	if ($listonly) {
		#$sql = "SELECT TargetBuild FROM [TestResults].[dbo].[GetParityBuildsWithCISmokeTestResults] ('$build', '') ORDER BY TargetBuild DESC";
		if ( "$testtype" eq "ci" ) {
			$sql = "SELECT * FROM [TestResults].[dbo].[GetParityBuildsWithCISmokeTestResults] ('$build', '') ORDER BY TargetBuild DESC";
		} else {
			$sql = "SELECT * FROM [TestResults].[dbo].[GetParityBuildsWithDemandSmokeTestResults] ('$build', '') ORDER BY TargetBuild DESC";
		}
	} else {
		#$sql = "SELECT TargetBuild, TargetRelease, State, AgentPlatform, AgentDescription, PassResultCount, FailResultCountExpected, FailResultCountUnexpected, DetailsReportURL FROM [TestResults].[dbo].[GetParityCISmokeTestStatusWithExpectedFailures] ('$build', '') ORDER BY TargetBuild DESC, AgentPlatform DESC";
		if ( "$testtype" eq "ci" ) {
			$sql = "SELECT TargetBuild, TargetRelease, State, AgentPlatform, AgentDescription, PassResultCount, FailResultCountExpected, FailResultCountUnexpected, DetailsReportURL FROM [TestResults].[dbo].[GetParityCISmokeTestStatusWithExpectedFailures] ('$build', '') ORDER BY TargetBuild DESC, AgentPlatform DESC";
		} else {
			$sql = "SELECT TargetBuild, TargetRelease, State, AgentPlatform, AgentDescription, PassResultCount, FailResultCountExpected, FailResultCountUnexpected, DetailsReportURL FROM [TestResults].[dbo].[GetParityDemandSmokeTestStatusWithExpectedFailures] ('$build', '') ORDER BY TargetBuild DESC, AgentPlatform DESC";
		}
	}
} else {
	$sql = "SELECT * FROM GetParityCISmokeTestStatus";
}

if ($DEBUG) {
	open(DEBUGOUT, ">>$debuglog");
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: SQL query - $sql\n";
	print DEBUGOUT "ENV{REMOTE_USER} is \"$ENV{REMOTE_USER}\"\n";
	#close(DEBUGOUT);
}

# | RunID| RunName| TargetBuild | TargetRelease| State | AgentPlatform| AgentDescription| PassResultCount| FailResultCount| DetailsReportURL
# | 161351f573782b1a3a1283aad0e38c1e | 7.0.1.440 - 08/14/12 - 15:15 | 7.0.1.440 | 7.0.1 | Completed | MAC | Mac Snow Leopard (10.7) 64-Bit Server | 1 | 7 | http://testtrack/ReportServer_TestTrack?%2fQA%2fTest+Results%2fTest+Run+-+Details&rs:Command=Render&TestRun=161351f573782b1a3a1283aad0e38c1e
#$sql = "SELECT TargetBuild, RunName, State, AgentPlatform, AgentDescription, PassResultCount, FailResultCount, DetailsReportURL FROM [TestResults].[dbo].[GetParityCISmokeTestStatus] ('$build', '')";

# Prepare the statement.
if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: call sth = dbh->prepare($sql)\n";
}
$sth = $dbh->prepare($sql) or die "Can't prepare statement: $DBI::errstr";
if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: endcall sth = dbh->prepare($sql)\n";
}
# Execute the statement.
if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: call sth = execute()\n";
}
$sth->execute();
if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: enddall sth = execute()\n";
}

if ($listonly) {
	open(TMPSORT, ">/tmp/tmpsort$$") or die "Error opening /tmp/tmpsort$$";
}

# Fetch and display the result set value.
$count=1;
open(DBG, ">>/tmp/sqlsmoke.out");
print DBG "listonly is \"$listonly\"\n";
if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: call row = sth->fetchrow_array\n";
}
while ( @row = $sth->fetchrow_array ) {
	if ($listonly) {
		# Only add non-ci build versions for these people with the ci query
		next if ( "$testtype" eq "ci" and $row[0] !~ m/\-/ and $ENV{REMOTE_USER} ne "sdaigle" and $ENV{REMOTE_USER} ne "build" and $ENV{REMOTE_USER} ne "zroot" );
		print TMPSORT "$row[0]\n";
		#print "$row[0]\n";
		next;
	}

	if ( $ARGV[2] ne "ALL" ) {
		next if ( "@row" !~ m/$ARGV[2]\ / );
	}
	print DBG "row array is \"@row\"\n";
	print DBG "ARGV[1] is \"$ARGV[1]\"\n";
	$pass_color="green";
	$unexpected_fail_color="red";
	$expected_fail_color="darkgoldenrod";
	$color="odd";
	$color="even" if ( $count % 2 );
	if ( "@row" =~ m/$ARGV[1]\ / ) {
		print "<tr class=\"$color\">\n";
		print "<td>$row[0]</td>\n"; # Target Build
		print "<td>$row[1]</td>\n"; # Target Release

		#$in_progress_gif="spinner.gif"; # 16x16
		$in_progress_gif="rotating_arrow.gif"; # 20x21

		# State
		# Current States are: Scheduled, Queued, Initializing,
		#   In Progress, Failed,  Completed,  Completed With Error
		if ( $row[2] =~ m/Initializing/ ) {
			$image="yellow_spinner.gif";
		} elsif ( $row[2] =~ m/In Progress/ ) {
			$image="blue_spinner.gif";
		} elsif ( $row[2] =~ m/Scheduled/ ) {
			if ( $color eq "odd" ) {
				$image="scheduled_bground_white.jpg";
			} else {
				$image="scheduled_bground_tan.jpg";
			}
		} elsif ( $row[2] =~ m/Queued/ ) {
			if ( $color eq "odd" ) {
				$image="InQueue_white.jpg";
			} else {
				$image="InQueue_tan.jpg";
			}
		# Used for testing
		#} elsif ( $row[2] =~ m/Completed/ and $row[3] eq "MAC" ) {
			#if ( $color eq "odd" ) {
				#$image="InQueue_white.jpg";
			#} else {
				#$image="InQueue_tan.jpg";
			#}
		} else {
			$image="none";
		}
		if ( $image eq "none" ) {
			print "<td>$row[2]</td>";
		} else {
			print "<td>$row[2] <img src=\"https://eng.bit9.local/$image\" width=\"12\" height=\"12\" border=\"0\"/></td>\n";
		}

		print "<td>$row[3]</td>\n"; # Agent Platform
		print "<td>$row[4]</td>\n"; # Agent Description

		# Make success colors
		$stat_color="black";
		if ( $row[5] > 0 ) {
			$stat_color="$pass_color";
		}
		$string="<td><b><font color=\"$stat_color\">$row[5]</b></td>";
		print "$string\n"; # Pass Result Count

		# Make failure colors
		$stat_color="black";
		if ( $row[6] > 0 ) {
			$stat_color="$expected_fail_color";
		}
		$string="<td><b><font color=\"$stat_color\">$row[6]</b></td>";
		print "$string\n"; # Fail Result Count

		# Make failure colors
		$stat_color="black";
		if ( $row[7] > 0 ) {
			$stat_color="$unexpected_fail_color";
		}
		$string="<td><b><font color=\"$stat_color\">$row[7]</b></td>";
		print "$string\n"; # Fail Result Count

		print "<td><a href =\"$row[8]\">View Report</a></td>\n"; # DetailsReportURL
		$count++;
	}
}
close(DBG);

if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: endcall row = sth->fetchrow_array\n";
}

if ($listonly) {
	close(TMPSORT);
	if ($DEBUG) {
		&getDateTime;
		print DEBUGOUT "$date: $thisscript: call sort -n -r -t. -k4 /tmp/tmpsort$$\n";
	}
	system("sort -n -r -t. -k4 /tmp/tmpsort$$ > /tmp/tmpsort2$$");
	open(TMPSORT, "/tmp/tmpsort2$$") or die "Error opening /tmp/tmpsort2$$";
	while (<TMPSORT>) {
		chomp;
		@last_num = split '\.', $_;
		$len = length($last_num[3]);
		# Skip build numbers with 5 digits or more; i.e 7.0.1.12345 and
		# do not have an # ascii string, i.e 7.0.1.1346-sdaigle-1143
		next if ( $len >= 5 and $last_num[3] !~ m/[a-z]/ );
		print "$_\n";
	}
	close(TMPSORT);
	unlink("/tmp/tmpsort$$");
	unlink("/tmp/tmpsort2$$");
	if ($DEBUG) {
		&getDateTime;
		print DEBUGOUT "$date: $thisscript: endcall sort -n -r -t. -k4 /tmp/tmpsort$$\n";
	}
}

if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: call dbh->disconnect\n";
}
# Disconnect the database from the database handle.
$dbh->disconnect;

if ($DEBUG) {
	&getDateTime;
	print DEBUGOUT "$date: $thisscript: endcall dbh->disconnect\n";
}

if ($DEBUG) {
	close(DEBUGOUT);
}
