#!/usr/bin/perl

# This script generates reports from querying the TestTrack SQL database for
# QA Smoketest builds. As with the buildstat.cgi, I use dynamically populated
# pulldown menus when the first pulldown menu is changed so when a Target Release
# is chosen in the first pulldown, the second pulldown will populate with available
# build numbers. This is done by calling the Jenkins API and querying all builds
# associated with the menu 1 pulldown version.

use File::Basename;
use IO::Handle;
use LWP::Simple;

sub getDateTime {
	$date=`date \"+%a %m-%d-%y %H:%M:%S\"`;
	chomp $date;
	$logprefix="$date - $ENV{REQUEST_URI} - $ENV{REMOTE_ADDR} - $ENV{REMOTE_USER}";
}

&getDateTime;

$TF1="/tmp/tf1$$";
$TF2="/tmp/tf2$$";
$RF1="/tmp/rf1$$"; # Used for reverse file of svn log output

#$ENV{JAVA_HOME}='/opt/java/jdk1.6.0_26';
$ENV{JAVA_HOME}='/usr/lib/jvm/jre-1.8.0-openjdk.x86_64';
$CURRENT_PATH="$ENV{PATH}";
$ENV{PATH}="$CURRENT_PATH:$ENV{JAVA_HOME}/bin";

$thisscript=basename($0);
($thisscript_strip_cgi_ext = $thisscript) =~ s/\.cgi//g;

# Just so I can copy the changes made in the smokestats-test .cgi to
# smokestats.cgi
if ( "$thisscript" eq "smokestats.cgi" ) {
	$perlscript="/var/www/cgi-bin/smokestats-sqlquery.pl";
} else {
	$perlscript="/var/www/cgi-bin/smokestats-sqlquery-test.pl";
}

# I only support the Das repo at this time
$thisdir=dirname($0);

##############################################################################
#
# Project specific settings. These will need to change over time.
#
$Bit9_Platform_Project1_Version="7.0.1";
$Bit9_Platform_Project2_Version="7.2.0";
$Bit9_Platform_Project3_Version="7.2.1";
##############################################################################

$DEBUG=1;
$debuglog="/tmp/$thisscript_strip_cgi_ext.out";
open(DEBUGOUT, ">>$debuglog") if ($DEBUG);
&getDateTime;
print DEBUGOUT "$logprefix\n";

&getDateTime;
print DEBUGOUT "$logprefix - $ENV{'REQUEST_METHOD'}\n" if ($DEBUG);

sub ReadParse {
	local (*in) = @_ if @_;
	local ($i, $loc, $key, $val);

	# Read in text
	if ($ENV{'REQUEST_METHOD'} eq "GET") {
		$in = $ENV{'QUERY_STRING'};
	} elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
		read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
	}

	@in = split(/&/,$in);

	foreach $i (0 .. $#in) {
		# Convert pluses to spaces
		$in[$i] =~ s/\+/ /g;

		# Split into key and value.
		($key, $val) = split(/=/,$in[$i],2); # splits on the first =.

		# Convert %XX from hex numbers to alphanumeric
		$key =~ s/%(..)/pack("c",hex($1))/ge;
		$val =~ s/%(..)/pack("c",hex($1))/ge;

		# Associate key and value
		$in{$key} .= "\0" if (defined($in{$key})); # \0 is the multiple separator
		$in{$key} .= $val;

		&getDateTime;
		print DEBUGOUT "$logprefix - AAA key is \"$key\"\n" if ($DEBUG);
		print DEBUGOUT "$logprefix - AAA val is \"$val\"\n" if ($DEBUG);

		if ( $key =~ /optone/ ) {
			$optone="$val";
			&getDateTime;
			print DEBUGOUT "$logprefix - optone is \"$optone\"\n" if ($DEBUG);
		} elsif ( $key =~ /opttwo/ ) {
			$opttwo="$val";
			&getDateTime;
			print DEBUGOUT "$logprefix - opttwo is \"$opttwo\"\n" if ($DEBUG);
		} elsif ( $key =~ /osPlatform/ ) {
			$osPlatform="$val";
			&getDateTime;
			print DEBUGOUT "$logprefix - osPlatform is \"$osPlatform\"\n" if ($DEBUG);
		} elsif ( $key =~ /argurl/ ) {
			$argurl="$val";
			&getDateTime;
			print DEBUGOUT "$logprefix - argurl is \"$argurl\"\n" if ($DEBUG);
		}
		&getDateTime;
		print DEBUGOUT "$logprefix - AAA opttwo is \"$opttwo\"\n" if ($DEBUG);
	}
}

sub DoPostAndArgs {
	#&ReadParse;

	#ZZZZ

	# Save original STDOUT. I do this so I have the option tweaking the
	# final output with sed or something.
	open (ORIGSTDOUT, ">&STDOUT") or die "Couldn't dup STDOUT: $!";
	open (STDOUT, ">> $TF1") or die "Can't open $TF1: $!";
	$stdout_restored="no";

	&getDateTime;
	print DEBUGOUT "$logprefix - BBB opttwo is \"$opttwo\"\n" if ($DEBUG);
	if ( "$opttwo" =~ m/\S+/ ) {
		$htmlend=0;
		&printReport;
		# optone would be the version #, i.e 7.0.1
		if ( $opttwo =~ m/ALL/ ) {
			$pass_arg="$optone";
		} else {
			$pass_arg="$opttwo";
		}

		if ( $optone =~ m/ci/ ) {
			$testtype="ci";
		} else {
			$testtype="demand";
		}

		# Close and re-open so perl script can log to same file
		close(DEBUGOUT) if ($DEBUG);
		if ( "$osPlatform" =~ m/ALL/ ) {
			&getDateTime;
			print DEBUGOUT "$logprefix - call $perlscript $testtype $pass_arg\n";
			system("perl $perlscript $testtype $pass_arg");
			&getDateTime;
			print DEBUGOUT "$logprefix - return $perlscript $testtype $pass_arg\n";
		} else {
			&getDateTime;
			print DEBUGOUT "$logprefix - call $perlscript $testtype $pass_arg $osPlatform\n";
			system("perl $perlscript $testtype $pass_arg $osPlatform");
			&getDateTime;
			print DEBUGOUT "$logprefix - return $perlscript $testtype $pass_arg $osPlatform\n";
		}
		open(DEBUGOUT, ">>$debuglog") if ($DEBUG);
		&getDateTime;
		print DEBUGOUT "$logprefix - osPlatform is \"$osPlatform\"\n" if ($DEBUG);
	} else {
		$htmlend=1;
		&doInitialHeader;
		print "<h2>Ooops</h2>\n";
		print "<font color=\"red\"><b>ERROR:</b></font> You need to choose a <b>Target Release</b> from the menu pulldown. Go back to main page and try again.<br>\n";
	}
	print<<EOF;
</tr></table>
	<p>
	<a href="https://eng-re.bit9.local/$thisscript_strip_cgi_ext">Back to Main Page</a>
EOF
	if ($htmlend) {
                print "</body></html>\n";
	}
	#&getDateTime;
	#print DEBUGOUT "$logprefix - (POST)\n";

	# Restore original STDOUT
	if ( $stdout_restored eq "no" ) {
		close STDOUT;
		open (STDOUT, ">&ORIGSTDOUT") or die "Can't restore STDOUT: $!";
		close ORIGSTDOUT;
	}
	# This is where you can modify strings in the final output
	system("cat $TF1 | sed \"s/AAAAA/NOTHING/g;s/BBBBB/NOTHING/g\"");
	unlink("$TF1");

	close(DEBUGOUT) if ($DEBUG);
	exit;

}

&ReadParse;

if ( $ENV{'REQUEST_METHOD'} eq "POST" ) {
	&getDateTime;
	print DEBUGOUT "$logprefix - (POST)\n";
	&DoPostAndArgs;
} elsif ( $argurl =~ m/./ ) {
	# Allow args passed in with url:
	# https://eng.bit9.local/smokestats?optone=7.0.1-demand&opttwo=7.0.1.1336&osPlatform=ALL&argurl=1
	&getDateTime;
	print DEBUGOUT "$logprefix - argurl passed in \"$argurl\"\n";
        &DoPostAndArgs;
}

# Get the smoketest version from the TestTrack SQL database instead of Jenkins
# Jenkins may have more versions and the only ones that should be in the menu
# pulldown are the ones available in SQL
# /var/www/cgi-bin/smokestats-sqlquery.pl 
&getDateTime;
print DEBUGOUT "$logprefix - call project1_ci_array $perlscript list\n";
@project1_ci_array=`perl $perlscript ci $Bit9_Platform_Project1_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project1_ci_array $perlscript list\n";
chomp(@project1_ci_array);
%seen=(); @unique_project1_ci_array = grep { ! $seen{$_} ++ } @project1_ci_array;
$numofelements=scalar(@unique_project1_ci_array);
$numofelements--;
$num=0;
@project1_ci_array=();
foreach $version (@unique_project1_ci_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project1_ci_array, $string);
}

if ( "@project1_ci_array" !~ m/./ ) {
	push(@project1_ci_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project1_ci_array, "\'ALL\', ");
}

&getDateTime;
#print DEBUGOUT "$logprefix - call project1_demand_array $perlscript list\n";
print DEBUGOUT "$logprefix - call perl $perlscript demand $Bit9_Platform_Project1_Version \"ALL\" list\n";
@project1_demand_array=`perl $perlscript demand $Bit9_Platform_Project1_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project1_demand_array $perlscript list\n";
chomp(@project1_demand_array);
%seen=(); @unique_project1_demand_array = grep { ! $seen{$_} ++ } @project1_demand_array;
$numofelements=scalar(@unique_project1_demand_array);
$numofelements--;
$num=0;
@project1_demand_array=();
foreach $version (@unique_project1_demand_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project1_demand_array, $string);
}

if ( "@project1_demand_array" !~ m/./ ) {
	push(@project1_demand_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project1_demand_array, "\'ALL\', ");
}

&getDateTime;
print DEBUGOUT "$logprefix - call project2_ci_array $perlscript list\n";
@project2_ci_array=`perl $perlscript ci $Bit9_Platform_Project2_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project2_ci_array $perlscript list\n";
chomp(@project2_ci_array);
%seen=(); @unique_project2_ci_array = grep { ! $seen{$_} ++ } @project2_ci_array;
$numofelements=scalar(@unique_project2_ci_array);
$numofelements--;
$num=0;
@project2_ci_array=();
foreach $version (@unique_project2_ci_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project2_ci_array, $string);
}

if ( "@project2_ci_array" !~ m/./ ) {
	push(@project2_ci_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project2_ci_array, "\'ALL\', ");
}

&getDateTime;
print DEBUGOUT "$logprefix - call project2_demand_array $perlscript list\n";
@project2_demand_array=`perl $perlscript demand $Bit9_Platform_Project2_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project2_demand_array $perlscript list\n";
chomp(@project2_demand_array);
%seen=(); @unique_project2_demand_array = grep { ! $seen{$_} ++ } @project2_demand_array;
$numofelements=scalar(@unique_project2_demand_array);
$numofelements--;
$num=0;
@project2_demand_array=();
foreach $version (@unique_project2_demand_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project2_demand_array, $string);
}

if ( "@project2_demand_array" !~ m/./ ) {
	push(@project2_demand_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project2_demand_array, "\'ALL\', ");
}

&getDateTime;
print DEBUGOUT "$logprefix - call project3_ci_array $perlscript list\n";
@project3_ci_array=`perl $perlscript ci $Bit9_Platform_Project3_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project3_ci_array $perlscript list\n";
chomp(@project3_ci_array);
%seen=(); @unique_project3_ci_array = grep { ! $seen{$_} ++ } @project3_ci_array;
$numofelements=scalar(@unique_project3_ci_array);
$numofelements--;
$num=0;
@project3_ci_array=();
foreach $version (@unique_project3_ci_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project3_ci_array, $string);
}

if ( "@project3_ci_array" !~ m/./ ) {
	push(@project3_ci_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project3_ci_array, "\'ALL\', ");
}

&getDateTime;
print DEBUGOUT "$logprefix - call project3_demand_array $perlscript list\n";
@project3_demand_array=`perl $perlscript demand $Bit9_Platform_Project3_Version \"ALL\" list`;
&getDateTime;
print DEBUGOUT "$logprefix - return project3_demand_array $perlscript list\n";
chomp(@project3_demand_array);
%seen=(); @unique_project3_demand_array = grep { ! $seen{$_} ++ } @project3_demand_array;
$numofelements=scalar(@unique_project3_demand_array);
$numofelements--;
$num=0;
@project3_demand_array=();
foreach $version (@unique_project3_demand_array) {
	if ($num >= $numofelements) {
		$string = "'" . "$version" . "'";
	} else {
		$string = "'" . "$version" . "',";
	}
	$num++;
	push(@project3_demand_array, $string);
}

if ( "@project3_demand_array" !~ m/./ ) {
	push(@project3_demand_array, "\'SQL Database Query Failed. Notify Administrator\'");
} else {
	unshift(@project3_demand_array, "\'ALL\', ");
}

sub printReport {
	# Do not put this line in the print EOF section
	print "Content-type:text/html\r\n\r\n";

print<<EOF;

<html>
<head>
<title>Automated QA Smoketest Statistics</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"/>
<link rel="stylesheet" href="smoketests.css" type="text/css"/>

</head>
<body>

<p>
<img src="https://eng-re.bit9.local/Smoketest_Stats.jpg" width="600" height=160>
<img src="https://eng-re.bit9.local/smoketesting.jpg" width="160" height=160>
<!--img src="https://eng-re.bit9.local/stormclouds.gif" width="160" height=160-->
</p>

<p>

<!-- From http://www.kryogenix.org/code/browser/sorttable -->
<script src="sorttable.js"></script>

<h2>Automated QA Smoketest Statistics from TestTrack Database</h2>

<h3>Click table descriptions to sort by column</h3>

<a href="https://eng-re.bit9.local/$thisscript_strip_cgi_ext">Back to Main Page</a><p>

<table class="sortable" border="1" width="90%">

<tr class=\"summaryperiod\">

<td width="20">Target Build</td>
<td width="5">Target Release</td>
<td width="15">State</td>
<td width="10">Agent Platform</td>
<td width="25">Agent Description</td>
<td width="5">Pass Result Count</td>
<td width="5">Expected Fail Result Count</td>
<td width="5">Unexpected Fail Result Count</td>
<td width="15">Detailed Report</td>

</tr>

EOF

}

sub doInitialHeader {
	# Do not put this line in the print EOF section
	print "Content-type:text/html\r\n\r\n";

print<<EOF;

<html>
<head>
<title>Automated QA Smoketest Statistics</title>
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"/>
<link rel="stylesheet" href="smoketests.css" type="text/css"/>

</head>
<body>

<p><img src="https://eng-re.bit9.local/Smoketest_Stats.jpg" width="600" height=160>
<img src="https://eng-re.bit9.local/smoketesting.jpg" width="160" height=160></p>
<p>

EOF
}

sub doHeader {
	&doInitialHeader;

print<<EOF;

<!-- From http://javascript.about.com/library/bl3drop.htm -->
<script type="text/javascript">
function setOptions(chosen, selbox) {

        //document.writeln(chosen)

	if (chosen == "7.0.1-ci") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project1_ci_array);
		}
	} else if (chosen == "7.0.1-demand") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project1_demand_array);
		}
	} else if (chosen == "7.2.0-ci") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project2_ci_array);
		}
	} else if (chosen == "7.2.0-demand") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project2_demand_array);
		}
	} else if (chosen == "7.2.1-ci") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project3_ci_array);
		}
	} else if (chosen == "7.2.1-demand") {
		if (typeof window.version === 'undefined') {
			window.version = new Array(@project3_demand_array);
		}
	}

	// Chome dupes menus so I need to know what browser is being used
	var str = navigator.appVersion;

        selbox.options.length = 0;
        if (chosen == " ") {
		location.reload(true);
                selbox.options[selbox.options.length] = new Option('Please select Target Release above first',' ');
        }

	for (i = 0; i < window.version.length; i++) {
		selbox.options[selbox.options.length] = new Option(window.version[i],window.version[i]);
	}
}

function optoneMenu() {
  setOptions(document.myform.optone.options[document.myform.optone.selectedIndex].value,document.myform.opttwo);
  //location.reload(); 
}

function opttwoMenu() {
  setOptions(document.myform.opttwo.options[document.myform.opttwo.selectedIndex].value);
}

</script>
<div class="section">
<table border="1" width="90%">

<form name="myform" action="https://eng.bit9.local/$thisscript_strip_cgi_ext" method="POST">

<div class="section">
<h2>Choose the Target Release you would like to generate a report for:</h2>

<select name="optone" size="1"
onchange="optoneMenu();">
<option value=" " selected="selected"> </option>
<option value="7.2.0-ci">7.2.0-ci</option>
<option value="7.2.0-demand">7.2.0-demand</option>
<option value="7.2.1-ci">7.2.1-ci</option>
<option value="7.2.1-demand">7.2.1-demand</option>
<option value="7.0.1-ci">7.0.1-ci</option>
<option value="7.0.1-demand">7.0.1-demand</option>
</select>: Pick <b>Target Release</b> to populate the Target Build pulldown menu below. Refresh browser if toggling between Target Releases to reset submenus<br><br>

<select name="opttwo" size="1"
onchange="opttwoMenu();">
<option value=" " selected="selected">Please select Target Release above first</option>
</select>: <b>Target Build</b><br> <br>

Platform Filters:<br>
<select name="osPlatform" size="1"
<option value=" " selected="ALL"> </option>
<option value="">ALL</option>
<option value="WINDOWS">WINDOWS</option>
<option value="LINUX-UBUNTU">LINUX-UBUNTU</option>
<option value="LINUX-REDHAT">LINUX-REDHAT</option>
<option value="MAC">MAC</option>
</select>
<br>
<br>

<input type="submit" value="Generate Report"><br>

</form>

EOF

}

# Print initial page
if ( $ENV{'REQUEST_METHOD'} ) {
	&doHeader;
	&ReadParse;
}

close(DEBUGOUT) if ($DEBUG);

exit 0;
